@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @file points/AttributeArray.h
 ///
@@ -51,6 +24,7 @@
 #include <tbb/atomic.h>
 
 #include <memory>
+#include <mutex>
 #include <string>
 #include <type_traits>
 
@@ -78,7 +52,7 @@ floatingPointToFixedPoint(const FloatT s)
     static_assert(std::is_unsigned<IntegerT>::value, "IntegerT must be unsigned");
     if (FloatT(0.0) > s) return std::numeric_limits<IntegerT>::min();
     else if (FloatT(1.0) <= s) return std::numeric_limits<IntegerT>::max();
-    return IntegerT(std::floor(s * FloatT(std::numeric_limits<IntegerT>::max())));
+    return IntegerT(s * FloatT(std::numeric_limits<IntegerT>::max()));
 }
 
 
@@ -127,7 +101,6 @@ public:
     enum Flag {
         TRANSIENT = 0x1,            /// by default not written to disk
         HIDDEN = 0x2,               /// hidden from UIs or iterators
-        OUTOFCORE = 0x4,            /// data not yet loaded from disk (deprecated flag as of ABI=5)
         CONSTANTSTRIDE = 0x8,       /// stride size does not vary in the array
         STREAMING = 0x10,           /// streaming mode collapses attributes when first accessed
         PARTIALREAD = 0x20          /// data has been partially read (compressed bytes is used)
@@ -152,15 +125,11 @@ public:
     using Ptr           = std::shared_ptr<AttributeArray>;
     using ConstPtr      = std::shared_ptr<const AttributeArray>;
 
-    using FactoryMethod = Ptr (*)(Index, Index, bool);
+    using FactoryMethod = Ptr (*)(Index, Index, bool, const Metadata*);
 
     template <typename ValueType, typename CodecType> friend class AttributeHandle;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     AttributeArray(): mPageHandle() { mOutOfCore = 0; }
-#else
-    AttributeArray(): mPageHandle() {}
-#endif
     virtual ~AttributeArray()
     {
         // if this AttributeArray has been partially read, zero the compressed bytes,
@@ -168,43 +137,23 @@ public:
         if (mFlags & PARTIALREAD)       mCompressedBytes = 0;
     }
 #if OPENVDB_ABI_VERSION_NUMBER >= 6
-    AttributeArray(const AttributeArray& rhs)
-        : mIsUniform(rhs.mIsUniform)
-        , mFlags(rhs.mFlags)
-        , mUsePagedRead(rhs.mUsePagedRead)
-        , mOutOfCore(rhs.mOutOfCore)
-        , mPageHandle()
-    {
-        if (mFlags & PARTIALREAD)       mCompressedBytes = rhs.mCompressedBytes;
-        else if (rhs.mPageHandle)       mPageHandle = rhs.mPageHandle->copy();
-    }
-    AttributeArray& operator=(const AttributeArray& rhs)
-    {
-        // if this AttributeArray has been partially read, zero the compressed bytes,
-        // so the page handle won't attempt to clean up invalid memory
-        if (mFlags & PARTIALREAD)       mCompressedBytes = 0;
-        mIsUniform = rhs.mIsUniform;
-        mFlags = rhs.mFlags;
-        mUsePagedRead = rhs.mUsePagedRead;
-        mOutOfCore = rhs.mOutOfCore;
-        if (mFlags & PARTIALREAD)       mCompressedBytes = rhs.mCompressedBytes;
-        else if (rhs.mPageHandle)       mPageHandle = rhs.mPageHandle->copy();
-        else                            mPageHandle.reset();
-        return *this;
-    }
+    AttributeArray(const AttributeArray& rhs);
+    AttributeArray& operator=(const AttributeArray& rhs);
 #else
     AttributeArray(const AttributeArray&) = default;
     AttributeArray& operator=(const AttributeArray&) = default;
 #endif
-    AttributeArray(AttributeArray&&) = default;
-    AttributeArray& operator=(AttributeArray&&) = default;
+    AttributeArray(AttributeArray&&) = delete;
+    AttributeArray& operator=(AttributeArray&&) = delete;
 
     /// Return a copy of this attribute.
-    /// @note This method is thread-safe.
     virtual AttributeArray::Ptr copy() const = 0;
 
-    /// Return an uncompressed copy of this attribute (will return a copy if not compressed).
-    /// @note This method is thread-safe.
+    /// Return a copy of this attribute.
+    /// @deprecated In-memory compression no longer supported, use AttributeArray::copy() instead.
+#ifndef _MSC_VER
+    OPENVDB_DEPRECATED
+#endif
     virtual AttributeArray::Ptr copyUncompressed() const = 0;
 
     /// Return the number of elements in this array.
@@ -257,7 +206,13 @@ public:
     /// @details If @a lock is non-null, the AttributeArray registry mutex
     /// has already been locked
     static Ptr create(const NamePair& type, Index length, Index stride = 1,
-        bool constantStride = true, const ScopedRegistryLock* lock = nullptr);
+        bool constantStride = true,
+        const Metadata* metadata = nullptr,
+        const ScopedRegistryLock* lock = nullptr);
+
+    static OPENVDB_DEPRECATED Ptr create(const NamePair& type, Index length,
+        Index stride, bool constantStride, const ScopedRegistryLock* lock);
+
     /// Return @c true if the given attribute type name is registered.
     static bool isRegistered(const NamePair& type, const ScopedRegistryLock* lock = nullptr);
     /// Clear the attribute type registry.
@@ -326,9 +281,6 @@ public:
     /// Compact the existing array to become uniform if all values are identical
     virtual bool compact() = 0;
 
-    /// @deprecated Previously this returned @c true if the array was compressed,
-    /// now it always returns @c false.
-    OPENVDB_DEPRECATED bool isCompressed() const { return false; }
     /// @deprecated Previously this compressed the attribute array, now it does nothing.
     // Windows does not allow base classes to be deprecated
 #ifndef _MSC_VER
@@ -431,6 +383,10 @@ private:
 #endif
 
 protected:
+#if OPENVDB_ABI_VERSION_NUMBER >= 7
+    AttributeArray(const AttributeArray& rhs, const tbb::spin_mutex::scoped_lock&);
+#endif
+
     /// @brief Specify whether this attribute has a constant stride or not.
     void setConstantStride(bool state);
 
@@ -449,9 +405,7 @@ protected:
     size_t mCompressedBytes = 0;
     uint8_t mFlags = 0;
     uint8_t mUsePagedRead = 0;
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     tbb::atomic<Index32> mOutOfCore; // interpreted as bool
-#endif
     compression::PageHandle::Ptr mPageHandle;
 
 #else // #if OPENVDB_ABI_VERSION_NUMBER < 6
@@ -623,9 +577,23 @@ public:
     /// Default constructor, always constructs a uniform attribute.
     explicit TypedAttributeArray(Index n = 1, Index strideOrTotalSize = 1, bool constantStride = true,
         const ValueType& uniformValue = zeroVal<ValueType>());
+#if OPENVDB_ABI_VERSION_NUMBER >= 7
     /// Deep copy constructor.
-    /// @note not thread-safe, use TypedAttributeArray::copy() to ensure thread-safety
+    /// @note This method is thread-safe (as of ABI=7) for concurrently reading from the
+    /// source attribute array while being deep-copied. Specifically, this means that the
+    /// attribute array being deep-copied can be out-of-core and safely loaded in one thread
+    /// while being copied using this copy-constructor in another thread.
+    /// It is not thread-safe for write.
+    TypedAttributeArray(const TypedAttributeArray&);
+    /// Deep copy constructor.
+    /// @deprecated Use copy-constructor without unused bool parameter
+    OPENVDB_DEPRECATED TypedAttributeArray(const TypedAttributeArray&, bool /*unused*/);
+#else
+    /// Deep copy constructor.
+    /// @note This method is not thread-safe for reading or writing, use
+    /// TypedAttributeArray::copy() to ensure thread-safety when reading concurrently.
     TypedAttributeArray(const TypedAttributeArray&, bool uncompress = false);
+#endif
     /// Deep copy assignment operator.
     /// @note this operator is thread-safe.
     TypedAttributeArray& operator=(const TypedAttributeArray&);
@@ -640,12 +608,14 @@ public:
     /// @note This method is thread-safe.
     AttributeArray::Ptr copy() const override;
 
-    /// Return an uncompressed copy of this attribute (will just return a copy if not compressed).
+    /// Return a copy of this attribute.
     /// @note This method is thread-safe.
-    AttributeArray::Ptr copyUncompressed() const override;
+    /// @deprecated In-memory compression no longer supported, use AttributeArray::copy() instead.
+    OPENVDB_DEPRECATED AttributeArray::Ptr copyUncompressed() const override;
 
     /// Return a new attribute array of the given length @a n and @a stride with uniform value zero.
-    static Ptr create(Index n, Index strideOrTotalSize = 1, bool constantStride = true);
+    static Ptr create(Index n, Index strideOrTotalSize = 1, bool constantStride = true,
+        const Metadata* metadata = nullptr);
 
     /// Cast an AttributeArray to TypedAttributeArray<T>
     static TypedAttributeArray& cast(AttributeArray& attributeArray);
@@ -710,30 +680,30 @@ public:
     /// Return the number of bytes of memory used by this attribute.
     size_t memUsage() const override;
 
-    /// Return the value at index @a n (assumes uncompressed and in-core)
+    /// Return the value at index @a n (assumes in-core)
     ValueType getUnsafe(Index n) const;
     /// Return the value at index @a n
     ValueType get(Index n) const;
-    /// Return the @a value at index @a n (assumes uncompressed and in-core)
+    /// Return the @a value at index @a n (assumes in-core)
     template<typename T> void getUnsafe(Index n, T& value) const;
     /// Return the @a value at index @a n
     template<typename T> void get(Index n, T& value) const;
 
     /// Non-member equivalent to getUnsafe() that static_casts array to this TypedAttributeArray
-    /// (assumes uncompressed and in-core)
+    /// (assumes in-core)
     static ValueType getUnsafe(const AttributeArray* array, const Index n);
 
-    /// Set @a value at the given index @a n (assumes uncompressed and in-core)
+    /// Set @a value at the given index @a n (assumes in-core)
     void setUnsafe(Index n, const ValueType& value);
     /// Set @a value at the given index @a n
     void set(Index n, const ValueType& value);
-    /// Set @a value at the given index @a n (assumes uncompressed and in-core)
+    /// Set @a value at the given index @a n (assumes in-core)
     template<typename T> void setUnsafe(Index n, const T& value);
     /// Set @a value at the given index @a n
     template<typename T> void set(Index n, const T& value);
 
     /// Non-member equivalent to setUnsafe() that static_casts array to this TypedAttributeArray
-    /// (assumes uncompressed and in-core)
+    /// (assumes in-core)
     static void setUnsafe(AttributeArray* array, const Index n, const ValueType& value);
 
     /// Set value at given index @a n from @a sourceIndex of another @a sourceArray
@@ -824,10 +794,14 @@ protected:
 private:
     friend class ::TestAttributeArray;
 
+#if OPENVDB_ABI_VERSION_NUMBER >= 7
+    TypedAttributeArray(const TypedAttributeArray&, const tbb::spin_mutex::scoped_lock&);
+#endif
+
     /// Load data from memory-mapped file.
     inline void doLoad() const;
     /// Load data from memory-mapped file (unsafe as this function is not protected by a mutex).
-    /// @param compression if true, loading previously compressed data will re-compressed it
+    /// @param compression parameter no longer used
     inline void doLoadUnsafe(const bool compression = true) const;
     /// Compress in-core data assuming mutex is locked
     inline bool compressUnsafe();
@@ -849,11 +823,12 @@ private:
     void deallocate();
 
     /// Helper function for use with registerType()
-    static AttributeArray::Ptr factory(Index n, Index strideOrTotalSize, bool constantStride) {
-        return TypedAttributeArray::create(n, strideOrTotalSize, constantStride);
+    static AttributeArray::Ptr factory(Index n, Index strideOrTotalSize, bool constantStride,
+        const Metadata* metadata) {
+        return TypedAttributeArray::create(n, strideOrTotalSize, constantStride, metadata);
     }
 
-    static tbb::atomic<const NamePair*> sTypeName;
+    static std::unique_ptr<const NamePair> sTypeName;
     std::unique_ptr<StorageType[]>      mData;
     Index                               mSize;
     Index                               mStrideOrTotalSize;
@@ -1167,7 +1142,7 @@ void AttributeArray::copyValues(const AttributeArray& sourceArray, const IterT&
 // TypedAttributeArray implementation
 
 template<typename ValueType_, typename Codec_>
-tbb::atomic<const NamePair*> TypedAttributeArray<ValueType_, Codec_>::sTypeName;
+std::unique_ptr<const NamePair> TypedAttributeArray<ValueType_, Codec_>::sTypeName;
 
 
 template<typename ValueType_, typename Codec_>
@@ -1198,9 +1173,23 @@ TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(
 }
 
 
+#if OPENVDB_ABI_VERSION_NUMBER >= 7
+template<typename ValueType_, typename Codec_>
+TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs)
+    : TypedAttributeArray(rhs, tbb::spin_mutex::scoped_lock(rhs.mMutex))
+{
+}
+
+
+template<typename ValueType_, typename Codec_>
+TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs,
+    const tbb::spin_mutex::scoped_lock& lock)
+    : AttributeArray(rhs, lock)
+#else
 template<typename ValueType_, typename Codec_>
 TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs, bool)
     : AttributeArray(rhs)
+#endif
     , mSize(rhs.mSize)
     , mStrideOrTotalSize(rhs.mStrideOrTotalSize)
 #if OPENVDB_ABI_VERSION_NUMBER < 6
@@ -1243,10 +1232,11 @@ template<typename ValueType_, typename Codec_>
 inline const NamePair&
 TypedAttributeArray<ValueType_, Codec_>::attributeType()
 {
-    if (sTypeName == nullptr) {
-        NamePair* s = new NamePair(typeNameAsString<ValueType>(), Codec::name());
-        if (sTypeName.compare_and_swap(s, nullptr) != nullptr) delete s;
-    }
+    static std::once_flag once;
+    std::call_once(once, []()
+    {
+        sTypeName.reset(new NamePair(typeNameAsString<ValueType>(), Codec::name()));
+    });
     return *sTypeName;
 }
 
@@ -1277,9 +1267,14 @@ TypedAttributeArray<ValueType_, Codec_>::unregisterType()
 
 template<typename ValueType_, typename Codec_>
 inline typename TypedAttributeArray<ValueType_, Codec_>::Ptr
-TypedAttributeArray<ValueType_, Codec_>::create(Index n, Index stride, bool constantStride)
+TypedAttributeArray<ValueType_, Codec_>::create(Index n, Index stride, bool constantStride,
+    const Metadata* metadata)
 {
-    return Ptr(new TypedAttributeArray(n, stride, constantStride));
+    const TypedMetadata<ValueType>* typedMetadata = metadata ?
+        dynamic_cast<const TypedMetadata<ValueType>*>(metadata) : nullptr;
+
+    return Ptr(new TypedAttributeArray(n, stride, constantStride,
+        typedMetadata ? typedMetadata->value() : zeroVal<ValueType>()));
 }
 
 template<typename ValueType_, typename Codec_>
@@ -1306,7 +1301,9 @@ template<typename ValueType_, typename Codec_>
 AttributeArray::Ptr
 TypedAttributeArray<ValueType_, Codec_>::copy() const
 {
+#if OPENVDB_ABI_VERSION_NUMBER < 7
     tbb::spin_mutex::scoped_lock lock(mMutex);
+#endif
     return AttributeArray::Ptr(new TypedAttributeArray<ValueType, Codec>(*this));
 }
 
@@ -1315,8 +1312,7 @@ template<typename ValueType_, typename Codec_>
 AttributeArray::Ptr
 TypedAttributeArray<ValueType_, Codec_>::copyUncompressed() const
 {
-    tbb::spin_mutex::scoped_lock lock(mMutex);
-    return AttributeArray::Ptr(new TypedAttributeArray<ValueType, Codec>(*this, /*decompress = */true));
+    return this->copy();
 }
 
 
@@ -1660,11 +1656,7 @@ template<typename ValueType_, typename Codec_>
 bool
 TypedAttributeArray<ValueType_, Codec_>::isOutOfCore() const
 {
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     return mOutOfCore;
-#else
-    return (mFlags & OUTOFCORE);
-#endif
 }
 
 
@@ -1672,12 +1664,7 @@ template<typename ValueType_, typename Codec_>
 void
 TypedAttributeArray<ValueType_, Codec_>::setOutOfCore(const bool b)
 {
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     mOutOfCore = b;
-#else
-    if (b) mFlags = static_cast<uint8_t>(mFlags | OUTOFCORE);
-    else   mFlags = static_cast<uint8_t>(mFlags & ~OUTOFCORE);
-#endif
 }
 
 
@@ -1885,11 +1872,7 @@ TypedAttributeArray<ValueType_, Codec_>::writeMetadata(std::ostream& os, bool ou
         OPENVDB_THROW(IoError, "Cannot write out a partially-read AttributeArray.");
     }
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     uint8_t flags(mFlags);
-#else
-    uint8_t flags(mFlags & uint8_t(~OUTOFCORE));
-#endif
     uint8_t serializationFlags(0);
     Index size(mSize);
     Index stride(mStrideOrTotalSize);
@@ -2024,11 +2007,7 @@ TypedAttributeArray<ValueType_, Codec_>::doLoadUnsafe(const bool /*compression*/
 
     // clear all write and out-of-core flags
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     self->mOutOfCore = false;
-#else
-    self->mFlags &= uint8_t(~OUTOFCORE);
-#endif
 }
 
 
@@ -2037,7 +2016,7 @@ AttributeArray::AccessorBasePtr
 TypedAttributeArray<ValueType_, Codec_>::getAccessor() const
 {
     // use the faster 'unsafe' get and set methods as attribute handles
-    // ensure data is uncompressed and in-core when constructed
+    // ensure data is in-core when constructed
 
     return AccessorBasePtr(new AttributeArray::Accessor<ValueType_>(
         &TypedAttributeArray<ValueType_, Codec_>::getUnsafe,
@@ -2345,7 +2324,3 @@ AttributeArray& AttributeWriteHandle<ValueType, CodecType>::array()
 } // namespace openvdb
 
 #endif // OPENVDB_POINTS_ATTRIBUTE_ARRAY_HAS_BEEN_INCLUDED
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
