@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @file points/AttributeSet.cc
 
@@ -101,14 +74,19 @@ AttributeSet::AttributeSet(const AttributeSet& attrSet, Index arrayLength,
         lock = localLock.get();
     }
 
+    const MetaMap& meta = mDescr->getMetadata();
+    bool hasMetadata = meta.metaCount();
+
     for (const auto& namePos : mDescr->map()) {
         const size_t& pos = namePos.second;
+        Metadata::ConstPtr metadata;
+        if (hasMetadata)    metadata = meta["default:" + namePos.first];
         const AttributeArray* existingArray = attrSet.getConst(pos);
         const bool constantStride = existingArray->hasConstantStride();
         const Index stride = constantStride ? existingArray->stride() : existingArray->dataSize();
 
         AttributeArray::Ptr array = AttributeArray::create(mDescr->type(pos), arrayLength,
-            stride, constantStride, lock);
+            stride, constantStride, metadata.get(), lock);
 
         // transfer hidden and transient flags
         if (existingArray->isHidden())      array->setHidden(true);
@@ -130,10 +108,15 @@ AttributeSet::AttributeSet(const DescriptorPtr& descr, Index arrayLength,
         lock = localLock.get();
     }
 
+    const MetaMap& meta = mDescr->getMetadata();
+    bool hasMetadata = meta.metaCount();
+
     for (const auto& namePos : mDescr->map()) {
         const size_t& pos = namePos.second;
+        Metadata::ConstPtr metadata;
+        if (hasMetadata)    metadata = meta["default:" + namePos.first];
         mAttrs[pos] = AttributeArray::create(mDescr->type(pos), arrayLength,
-            /*stride=*/1, /*constantStride=*/true, lock);
+            /*stride=*/1, /*constantStride=*/true, metadata.get(), lock);
     }
 }
 
@@ -264,6 +247,20 @@ AttributeSet::groupIndex(const size_t offset) const
     return mDescr->groupIndex(offset);
 }
 
+std::vector<size_t>
+AttributeSet::groupAttributeIndices() const
+{
+    std::vector<size_t> indices;
+
+    for (const auto& namePos : mDescr->map()) {
+        const AttributeArray* array = this->getConst(namePos.first);
+        if (isGroup(*array)) {
+            indices.push_back(namePos.second);
+        }
+    }
+
+    return indices;
+}
 
 bool
 AttributeSet::isShared(size_t pos) const
@@ -290,7 +287,7 @@ AttributeSet::appendAttribute(  const Name& name,
                                 const NamePair& type,
                                 const Index strideOrTotalSize,
                                 const bool constantStride,
-                                Metadata::Ptr defaultValue)
+                                const Metadata* defaultValue)
 {
     Descriptor::Ptr descriptor = mDescr->duplicateAppend(name, type);
 
@@ -300,13 +297,14 @@ AttributeSet::appendAttribute(  const Name& name,
     // extract the index from the descriptor
     const size_t pos = descriptor->find(name);
 
-    return this->appendAttribute(*mDescr, descriptor, pos, strideOrTotalSize, constantStride);
+    return this->appendAttribute(*mDescr, descriptor, pos, strideOrTotalSize, constantStride, defaultValue);
 }
 
 
 AttributeArray::Ptr
 AttributeSet::appendAttribute(  const Descriptor& expected, DescriptorPtr& replacement,
                                 const size_t pos, const Index strideOrTotalSize, const bool constantStride,
+                                const Metadata* defaultValue,
                                 const AttributeArray::ScopedRegistryLock* lock)
 {
     // ensure the descriptor is as expected
@@ -329,7 +327,8 @@ AttributeSet::appendAttribute(  const Descriptor& expected, DescriptorPtr& repla
     // append the new array
 
     AttributeArray::Ptr array = AttributeArray::create(
-        type, arrayLength, strideOrTotalSize, constantStride, lock);
+        type, arrayLength, strideOrTotalSize, constantStride,
+        defaultValue, lock);
 
     // if successful, update Descriptor and append the created array
 
@@ -341,6 +340,71 @@ AttributeSet::appendAttribute(  const Descriptor& expected, DescriptorPtr& repla
 }
 
 
+// deprecated
+AttributeArray::Ptr
+AttributeSet::appendAttribute(  const Name& name,
+                                const NamePair& type,
+                                const Index strideOrTotalSize,
+                                const bool constantStride,
+                                Metadata::Ptr defaultValue)
+{
+    return this->appendAttribute(name, type, strideOrTotalSize,
+        constantStride, defaultValue.get());
+}
+
+
+// deprecated
+AttributeArray::Ptr
+AttributeSet::appendAttribute(  const Descriptor& expected, DescriptorPtr& replacement,
+                                const size_t pos, const Index strideOrTotalSize,
+                                const bool constantStride,
+                                const AttributeArray::ScopedRegistryLock* lock)
+{
+    return this->appendAttribute(expected, replacement, pos, strideOrTotalSize,
+        constantStride, nullptr, lock);
+}
+
+
+AttributeArray::Ptr
+AttributeSet::removeAttribute(const Name& name)
+{
+    const size_t pos = this->find(name);
+    if (pos == INVALID_POS) return AttributeArray::Ptr();
+    return this->removeAttribute(pos);
+}
+
+
+AttributeArray::Ptr
+AttributeSet::removeAttribute(const size_t pos)
+{
+    if (pos >= mAttrs.size())     return AttributeArray::Ptr();
+
+    assert(mAttrs[pos]);
+    AttributeArray::Ptr array;
+    std::swap(array, mAttrs[pos]);
+    assert(array);
+
+    // safely drop the attribute and update the descriptor
+    std::vector<size_t> toDrop{pos};
+    this->dropAttributes(toDrop);
+
+    return array;
+}
+
+
+AttributeArray::Ptr
+AttributeSet::removeAttributeUnsafe(const size_t pos)
+{
+    if (pos >= mAttrs.size())     return AttributeArray::Ptr();
+
+    assert(mAttrs[pos]);
+    AttributeArray::Ptr array;
+    std::swap(array, mAttrs[pos]);
+
+    return array;
+}
+
+
 void
 AttributeSet::dropAttributes(const std::vector<size_t>& pos)
 {
@@ -959,9 +1023,24 @@ AttributeSet::Descriptor::hasGroup(const Name& group) const
 }
 
 void
-AttributeSet::Descriptor::setGroup(const Name& group, const size_t offset)
+AttributeSet::Descriptor::setGroup(const Name& group, const size_t offset,
+    const bool checkValidOffset)
 {
-    if (!validName(group))  throw RuntimeError("Group name contains invalid characters - " + group);
+    if (!validName(group)) {
+        throw RuntimeError("Group name contains invalid characters - " + group);
+    }
+    if (checkValidOffset) {
+        // check offset is not out-of-range
+        if (offset >= this->availableGroups()) {
+            throw RuntimeError("Group offset is out-of-range - " + group);
+        }
+        // check offset is not already in use
+        for (const auto& namePos : mGroupMap) {
+            if (namePos.second == offset) {
+                throw RuntimeError("Group offset is already in use - " + group);
+            }
+        }
+    }
 
     mGroupMap[group] = offset;
 }
@@ -1056,14 +1135,11 @@ AttributeSet::Descriptor::groupIndex(const Name& group) const
     return this->groupIndex(offset);
 }
 
-
 AttributeSet::Descriptor::GroupIndex
 AttributeSet::Descriptor::groupIndex(const size_t offset) const
 {
     // extract all attribute array group indices
 
-    const size_t GROUP_BITS = sizeof(GroupType) * CHAR_BIT;
-
     std::vector<size_t> groups;
     for (const auto& namePos : mNameMap) {
         if (mTypes[namePos.second] == GroupAttributeArray::attributeType()) {
@@ -1071,15 +1147,145 @@ AttributeSet::Descriptor::groupIndex(const size_t offset) const
         }
     }
 
-    if (offset >= groups.size() * GROUP_BITS) {
+    if (offset >= groups.size() * this->groupBits()) {
         OPENVDB_THROW(LookupError, "Out of range group offset - " << offset << ".")
     }
 
     // adjust relative offset to find offset into the array vector
 
     std::sort(groups.begin(), groups.end());
-    return Util::GroupIndex(groups[offset / GROUP_BITS],
-                static_cast<uint8_t>(offset % GROUP_BITS));
+    return Util::GroupIndex(groups[offset / this->groupBits()],
+                static_cast<uint8_t>(offset % this->groupBits()));
+}
+
+size_t
+AttributeSet::Descriptor::availableGroups() const
+{
+    // the number of group attributes * number of bits per group
+
+    const size_t groupAttributes =
+        this->count(GroupAttributeArray::attributeType());
+
+    return groupAttributes * this->groupBits();
+}
+
+size_t
+AttributeSet::Descriptor::unusedGroups() const
+{
+    // compute total slots (one slot per bit of the group attributes)
+
+    const size_t availableGroups = this->availableGroups();
+
+    if (availableGroups == 0)   return 0;
+
+    // compute slots in use
+
+    const size_t usedGroups = mGroupMap.size();
+
+    return availableGroups - usedGroups;
+}
+
+bool
+AttributeSet::Descriptor::canCompactGroups() const
+{
+    // can compact if more unused groups than in one group attribute array
+
+    return this->unusedGroups() >= this->groupBits();
+}
+
+size_t
+AttributeSet::Descriptor::unusedGroupOffset(size_t hint) const
+{
+    // all group offsets are in use
+
+    if (unusedGroups() == size_t(0)) {
+        return std::numeric_limits<size_t>::max();
+    }
+
+    // build a list of group indices
+
+    std::vector<size_t> indices;
+    indices.reserve(mGroupMap.size());
+    for (const auto& namePos : mGroupMap) {
+        indices.push_back(namePos.second);
+    }
+
+    std::sort(indices.begin(), indices.end());
+
+    // return hint if not already in use
+
+    if (hint != std::numeric_limits<Index>::max() &&
+        hint < availableGroups() &&
+        std::find(indices.begin(), indices.end(), hint) == indices.end()) {
+        return hint;
+    }
+
+    // otherwise return first index not present
+
+    size_t offset = 0;
+    for (const size_t& index : indices) {
+        if (index != offset)     break;
+        offset++;
+    }
+
+    return offset;
+}
+
+// deprecated
+size_t
+AttributeSet::Descriptor::nextUnusedGroupOffset() const
+{
+    return this->unusedGroupOffset();
+}
+
+bool
+AttributeSet::Descriptor::requiresGroupMove(Name& sourceName,
+    size_t& sourceOffset, size_t& targetOffset) const
+{
+    targetOffset = this->unusedGroupOffset();
+
+    for (const auto& namePos : mGroupMap) {
+
+        // move only required if source comes after the target
+
+        if (namePos.second >= targetOffset) {
+            sourceName = namePos.first;
+            sourceOffset = namePos.second;
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool
+AttributeSet::Descriptor::groupIndexCollision(const Descriptor& rhs) const
+{
+    const auto& groupMap = this->groupMap();
+    const auto& otherGroupMap = rhs.groupMap();
+
+    // iterate both group maps at the same time and find any keys that occur
+    // in both maps and test their values for equality
+
+    auto groupsIt1 = groupMap.cbegin();
+    auto groupsIt2 = otherGroupMap.cbegin();
+
+    while (groupsIt1 != groupMap.cend() && groupsIt2 != otherGroupMap.cend()) {
+        if (groupsIt1->first < groupsIt2->first) {
+            ++groupsIt1;
+        } else if (groupsIt1->first > groupsIt2->first) {
+            ++groupsIt2;
+        } else {
+            if (groupsIt1->second != groupsIt2->second) {
+                return true;
+            } else {
+                ++groupsIt1;
+                ++groupsIt2;
+            }
+        }
+    }
+
+    return false;
 }
 
 bool
@@ -1204,7 +1410,3 @@ AttributeSet::Descriptor::read(std::istream& is)
 } // namespace points
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
