@@ -21,9 +21,6 @@
 #include <openvdb/Types.h>
 #include <openvdb/math/Transform.h>
 
-#include <boost/integer.hpp> // boost::int_t<N>::least
-#include <boost/scoped_array.hpp>
-
 #include <tbb/blocked_range.h>
 #include <tbb/parallel_for.h>
 #include <tbb/task_scheduler_init.h>
@@ -85,8 +82,13 @@ public:
     using ConstPtr = SharedPtr<const PointPartitioner>;
 
     using IndexType = PointIndexType;
-    using VoxelOffsetType = typename boost::int_t<1 + (3 * BucketLog2Dim)>::least;
-    using VoxelOffsetArray = boost::scoped_array<VoxelOffsetType>;
+
+    static constexpr Index bits = 1 + (3 * BucketLog2Dim);
+    // signed, so if bits is exactly 16, int32 is required
+    using VoxelOffsetType = typename std::conditional<(bits < 16),
+        int16_t, typename std::conditional<(bits < 32), int32_t, int64_t>::type>::type;
+
+    using VoxelOffsetArray = std::unique_ptr<VoxelOffsetType[]>;
 
     class IndexIterator;
 
@@ -161,11 +163,11 @@ private:
     PointPartitioner(const PointPartitioner&);
     PointPartitioner& operator=(const PointPartitioner&);
 
-    boost::scoped_array<IndexType>  mPointIndices;
+    std::unique_ptr<IndexType[]>    mPointIndices;
     VoxelOffsetArray                mVoxelOffsets;
 
-    boost::scoped_array<IndexType>  mPageOffsets;
-    boost::scoped_array<Coord>      mPageCoordinates;
+    std::unique_ptr<IndexType[]>    mPageOffsets;
+    std::unique_ptr<Coord[]>        mPageCoordinates;
     IndexType mPageCount;
     bool      mUsingCellCenteredTransform;
 }; // class PointPartitioner
@@ -272,9 +274,13 @@ struct CreateOrderedPointIndexArrayOp
 template<typename PointIndexType, Index BucketLog2Dim>
 struct VoxelOrderOp
 {
-    using VoxelOffsetType = typename boost::int_t<1 + (3 * BucketLog2Dim)>::least;
-    using VoxelOffsetArray = boost::scoped_array<VoxelOffsetType>;
-    using IndexArray = boost::scoped_array<PointIndexType>;
+    static constexpr Index bits = 1 + (3 * BucketLog2Dim);
+    // signed, so if bits is exactly 16, int32 is required
+    using VoxelOffsetType = typename std::conditional<(bits < 16),
+        int16_t, typename std::conditional<(bits < 32), int32_t, int64_t>::type>::type;
+
+    using VoxelOffsetArray = std::unique_ptr<VoxelOffsetType[]>;
+    using IndexArray = std::unique_ptr<PointIndexType[]>;
 
     VoxelOrderOp(IndexArray& indices, const IndexArray& pages,const VoxelOffsetArray& offsets)
         : mIndices(indices.get())
@@ -293,9 +299,9 @@ struct VoxelOrderOp
         const PointIndexType voxelCount = 1 << (3 * BucketLog2Dim);
 
         // allocate histogram buffers
-        boost::scoped_array<VoxelOffsetType> offsets(new VoxelOffsetType[pointCount]);
-        boost::scoped_array<PointIndexType> sortedIndices(new PointIndexType[pointCount]);
-        boost::scoped_array<PointIndexType> histogram(new PointIndexType[voxelCount]);
+        std::unique_ptr<VoxelOffsetType[]> offsets(new VoxelOffsetType[pointCount]);
+        std::unique_ptr<PointIndexType[]> sortedIndices(new PointIndexType[pointCount]);
+        std::unique_ptr<PointIndexType[]> histogram(new PointIndexType[voxelCount]);
 
         for (size_t n(range.begin()), N(range.end()); n != N; ++n) {
 
@@ -339,68 +345,13 @@ struct VoxelOrderOp
 }; // struct VoxelOrderOp
 
 
-template<typename PointArray, typename PointIndexType>
-struct LeafNodeOriginOp
-{
-    using IndexArray = boost::scoped_array<PointIndexType>;
-    using CoordArray = boost::scoped_array<Coord>;
-
-    LeafNodeOriginOp(CoordArray& coordinates,
-        const IndexArray& indices, const IndexArray& pages,
-        const PointArray& points, const math::Transform& m, int log2dim, bool cellCenteredTransform)
-        : mCoordinates(coordinates.get())
-        , mIndices(indices.get())
-        , mPages(pages.get())
-        , mPoints(&points)
-        , mXForm(m)
-        , mLog2Dim(log2dim)
-        , mCellCenteredTransform(cellCenteredTransform)
-    {
-    }
-
-    void operator()(const tbb::blocked_range<size_t>& range) const {
-
-        using PosType = typename PointArray::PosType;
-
-        const bool cellCentered = mCellCenteredTransform;
-        const int mask = ~((1 << mLog2Dim) - 1);
-        Coord ijk;
-        PosType pos;
-
-        for (size_t n = range.begin(), N = range.end(); n != N; ++n) {
-
-            mPoints->getPos(mIndices[mPages[n]], pos);
-
-            if (std::isfinite(pos[0]) && std::isfinite(pos[1]) && std::isfinite(pos[2])) {
-                ijk = cellCentered ? mXForm.worldToIndexCellCentered(pos) :
-                    mXForm.worldToIndexNodeCentered(pos);
-
-                ijk[0] &= mask;
-                ijk[1] &= mask;
-                ijk[2] &= mask;
-
-                mCoordinates[n] = ijk;
-            }
-        }
-    }
-
-    Coord                 * const mCoordinates;
-    PointIndexType  const * const mIndices;
-    PointIndexType  const * const mPages;
-    PointArray      const * const mPoints;
-    math::Transform         const mXForm;
-    int                     const mLog2Dim;
-    bool                    const mCellCenteredTransform;
-}; // struct LeafNodeOriginOp
-
-
 ////////////////////////////////////////
 
 
 template<typename T>
 struct Array
 {
-    using Ptr = SharedPtr<Array>;
+    using Ptr = std::unique_ptr<Array>;
 
     Array(size_t size) : mSize(size), mData(new T[size]) { }
 
@@ -413,15 +364,14 @@ struct Array
 
 private:
     size_t                  mSize;
-    boost::scoped_array<T>  mData;
+    std::unique_ptr<T[]>    mData;
 }; // struct Array
 
 
 template<typename PointIndexType>
 struct MoveSegmentDataOp
 {
-    using Segment = Array<PointIndexType>;
-    using SegmentPtr = typename Segment::Ptr;
+    using SegmentPtr = typename Array<PointIndexType>::Ptr;
 
     MoveSegmentDataOp(std::vector<PointIndexType*>& indexLists, SegmentPtr* segments)
         : mIndexLists(&indexLists[0]), mSegments(segments)
@@ -469,9 +419,9 @@ struct MergeBinsOp
 
     using IndexPair = std::pair<PointIndexType, PointIndexType>;
     using IndexPairList = std::deque<IndexPair>;
-    using IndexPairListPtr = SharedPtr<IndexPairList>;
+    using IndexPairListPtr = std::shared_ptr<IndexPairList>;
     using IndexPairListMap = std::map<Coord, IndexPairListPtr>;
-    using IndexPairListMapPtr = SharedPtr<IndexPairListMap>;
+    using IndexPairListMapPtr = std::shared_ptr<IndexPairListMap>;
 
     MergeBinsOp(IndexPairListMapPtr* bins,
         SegmentPtr* indexSegments,
@@ -589,9 +539,9 @@ struct BinPointIndicesOp
     using PosType = typename PointArray::PosType;
     using IndexPair = std::pair<PointIndexType, PointIndexType>;
     using IndexPairList = std::deque<IndexPair>;
-    using IndexPairListPtr = SharedPtr<IndexPairList>;
+    using IndexPairListPtr = std::shared_ptr<IndexPairList>;
     using IndexPairListMap = std::map<Coord, IndexPairListPtr>;
-    using IndexPairListMapPtr = SharedPtr<IndexPairListMap>;
+    using IndexPairListMapPtr = std::shared_ptr<IndexPairListMap>;
 
     BinPointIndicesOp(IndexPairListMapPtr* data,
         const PointArray& points,
@@ -706,14 +656,15 @@ struct BinPointIndicesOp
 template<typename PointIndexType>
 struct OrderSegmentsOp
 {
-    using IndexArray = boost::scoped_array<PointIndexType>;
+    using IndexArray = std::unique_ptr<PointIndexType[]>;
     using SegmentPtr = typename Array<PointIndexType>::Ptr;
 
-    OrderSegmentsOp(SegmentPtr* indexSegments, SegmentPtr* offestSegments,
-        IndexArray* pageOffsetArrays, Index binVolume)
+    OrderSegmentsOp(SegmentPtr* indexSegments, SegmentPtr* offsetSegments,
+        IndexArray* pageOffsetArrays, IndexArray* pageIndexArrays, Index binVolume)
         : mIndexSegments(indexSegments)
-        , mOffsetSegments(offestSegments)
+        , mOffsetSegments(offsetSegments)
         , mPageOffsetArrays(pageOffsetArrays)
+        , mPageIndexArrays(pageIndexArrays)
         , mBinVolume(binVolume)
     {
     }
@@ -730,7 +681,6 @@ struct OrderSegmentsOp
 
         IndexArray bucketIndices(new PointIndexType[maxSegmentSize]);
 
-
         for (size_t n = range.begin(), N = range.end(); n != N; ++n) {
 
             memset(bucketCounters.get(), 0, sizeof(PointIndexType) * bucketCountersSize);
@@ -754,13 +704,17 @@ struct OrderSegmentsOp
             pageOffsets.reset(new PointIndexType[nonemptyBucketCount + 1]);
             pageOffsets[0] = nonemptyBucketCount + 1; // stores array size in first element
 
+            IndexArray& pageIndices = mPageIndexArrays[n];
+            pageIndices.reset(new PointIndexType[nonemptyBucketCount]);
+
             // Compute bucket counter prefix sum
-            PointIndexType count = 0, idx = 1;
+            PointIndexType count = 0, idx = 0;
             for (size_t i = 0; i < bucketCountersSize; ++i) {
                 if (bucketCounters[i] != 0) {
-                    pageOffsets[idx] = bucketCounters[i];
+                    pageIndices[idx] = static_cast<PointIndexType>(i);
+                    pageOffsets[idx+1] = bucketCounters[i];
                     bucketCounters[i] = count;
-                    count += pageOffsets[idx];
+                    count += pageOffsets[idx+1];
                     ++idx;
                 }
             }
@@ -783,6 +737,7 @@ struct OrderSegmentsOp
     SegmentPtr * const mIndexSegments;
     SegmentPtr * const mOffsetSegments;
     IndexArray * const mPageOffsetArrays;
+    IndexArray * const mPageIndexArrays;
     Index        const mBinVolume;
 }; // struct OrderSegmentsOp
 
@@ -795,9 +750,9 @@ template<typename PointIndexType, typename VoxelOffsetType, typename PointArray>
 inline void binAndSegment(
     const PointArray& points,
     const math::Transform& xform,
-    boost::scoped_array<typename Array<PointIndexType>::Ptr>& indexSegments,
-    boost::scoped_array<typename Array<PointIndexType>::Ptr>& offsetSegments,
-    size_t& segmentCount,
+    std::unique_ptr<typename Array<PointIndexType>::Ptr[]>& indexSegments,
+    std::unique_ptr<typename Array<PointIndexType>::Ptr[]>& offsetSegments,
+    std::vector<Coord>& coords,
     const Index binLog2Dim,
     const Index bucketLog2Dim,
     VoxelOffsetType* voxelOffsets = nullptr,
@@ -805,15 +760,15 @@ inline void binAndSegment(
 {
     using IndexPair = std::pair<PointIndexType, PointIndexType>;
     using IndexPairList = std::deque<IndexPair>;
-    using IndexPairListPtr = SharedPtr<IndexPairList>;
+    using IndexPairListPtr = std::shared_ptr<IndexPairList>;
     using IndexPairListMap = std::map<Coord, IndexPairListPtr>;
-    using IndexPairListMapPtr = SharedPtr<IndexPairListMap>;
+    using IndexPairListMapPtr = std::shared_ptr<IndexPairListMap>;
 
     size_t numTasks = 1, numThreads = size_t(tbb::task_scheduler_init::default_num_threads());
     if (points.size() > (numThreads * 2)) numTasks = numThreads * 2;
     else if (points.size() > numThreads) numTasks = numThreads;
 
-    boost::scoped_array<IndexPairListMapPtr> bins(new IndexPairListMapPtr[numTasks]);
+    std::unique_ptr<IndexPairListMapPtr[]> bins(new IndexPairListMapPtr[numTasks]);
 
     using BinOp = BinPointIndicesOp<PointArray, PointIndexType, VoxelOffsetType>;
 
@@ -830,10 +785,10 @@ inline void binAndSegment(
         }
     }
 
-    std::vector<Coord> coords(uniqueCoords.begin(), uniqueCoords.end());
+    coords.assign(uniqueCoords.begin(), uniqueCoords.end());
     uniqueCoords.clear();
 
-    segmentCount = coords.size();
+    size_t segmentCount = coords.size();
 
     using SegmentPtr = typename Array<PointIndexType>::Ptr;
 
@@ -852,13 +807,16 @@ inline void partition(
     const PointArray& points,
     const math::Transform& xform,
     const Index bucketLog2Dim,
-    boost::scoped_array<PointIndexType>& pointIndices,
-    boost::scoped_array<PointIndexType>& pageOffsets,
+    std::unique_ptr<PointIndexType[]>& pointIndices,
+    std::unique_ptr<PointIndexType[]>& pageOffsets,
+    std::unique_ptr<Coord[]>& pageCoordinates,
     PointIndexType& pageCount,
-    boost::scoped_array<VoxelOffsetType>& voxelOffsets,
+    std::unique_ptr<VoxelOffsetType[]>& voxelOffsets,
     bool recordVoxelOffsets,
     bool cellCenteredTransform)
 {
+    using SegmentPtr = typename Array<PointIndexType>::Ptr;
+
     if (recordVoxelOffsets) voxelOffsets.reset(new VoxelOffsetType[points.size()]);
     else  voxelOffsets.reset();
 
@@ -868,29 +826,37 @@ inline void partition(
     //       (2^8)^3 = 256^3 voxel region.
 
 
-    size_t numSegments = 0;
+    std::vector<Coord> segmentCoords;
 
-    boost::scoped_array<typename Array<PointIndexType>::Ptr> indexSegments;
-    boost::scoped_array<typename Array<PointIndexType>::Ptr> offestSegments;
+    std::unique_ptr<SegmentPtr[]> indexSegments;
+    std::unique_ptr<SegmentPtr[]> offsetSegments;
 
     binAndSegment<PointIndexType, VoxelOffsetType, PointArray>(points, xform,
-        indexSegments, offestSegments, numSegments, binLog2Dim, bucketLog2Dim,
+        indexSegments, offsetSegments, segmentCoords, binLog2Dim, bucketLog2Dim,
             voxelOffsets.get(), cellCenteredTransform);
 
+    size_t numSegments = segmentCoords.size();
+
     const tbb::blocked_range<size_t> segmentRange(0, numSegments);
 
-    using IndexArray = boost::scoped_array<PointIndexType>;
-    boost::scoped_array<IndexArray> pageOffsetArrays(new IndexArray[numSegments]);
+    using IndexArray = std::unique_ptr<PointIndexType[]>;
+    std::unique_ptr<IndexArray[]> pageOffsetArrays(new IndexArray[numSegments]);
+    std::unique_ptr<IndexArray[]> pageIndexArrays(new IndexArray[numSegments]);
 
     const Index binVolume = 1u << (3u * binLog2Dim);
 
     tbb::parallel_for(segmentRange, OrderSegmentsOp<PointIndexType>
-        (indexSegments.get(), offestSegments.get(), pageOffsetArrays.get(), binVolume));
+        (indexSegments.get(), offsetSegments.get(),
+            pageOffsetArrays.get(), pageIndexArrays.get(), binVolume));
 
     indexSegments.reset();
 
+    std::vector<Index> segmentOffsets;
+    segmentOffsets.reserve(numSegments);
+
     pageCount = 0;
     for (size_t n = 0; n < numSegments; ++n) {
+        segmentOffsets.push_back(pageCount);
         pageCount += pageOffsetArrays[n][0] - 1;
     }
 
@@ -918,11 +884,51 @@ inline void partition(
     PointIndexType* index = pointIndices.get();
     for (size_t n = 0; n < numSegments; ++n) {
         indexArray.push_back(index);
-        index += offestSegments[n]->size();
+        index += offsetSegments[n]->size();
     }
 
+    // compute leaf node origin for each page
+
+    pageCoordinates.reset(new Coord[pageCount]);
+
     tbb::parallel_for(segmentRange,
-        MoveSegmentDataOp<PointIndexType>(indexArray, offestSegments.get()));
+        [&](tbb::blocked_range<size_t>& range)
+        {
+            for (size_t n = range.begin(); n < range.end(); n++)
+            {
+                Index segmentOffset = segmentOffsets[n];
+                PointIndexType* indices = pageIndexArrays[n].get();
+
+                const Coord& segmentCoord = segmentCoords[n];
+
+                // segment size stored in the first value of the offset array
+                const size_t segmentSize = pageOffsetArrays[n][0] - 1;
+                tbb::blocked_range<size_t> copyRange(0, segmentSize);
+                tbb::parallel_for(copyRange,
+                    [&](tbb::blocked_range<size_t>& r)
+                    {
+                        for (size_t i = r.begin(); i < r.end(); i++)
+                        {
+                            Index pageIndex = indices[i];
+                            Coord& ijk = pageCoordinates[segmentOffset+i];
+
+                            ijk[0] = pageIndex >> (2 * binLog2Dim);
+                            Index pageIndexModulo = pageIndex - (ijk[0] << (2 * binLog2Dim));
+                            ijk[1] = pageIndexModulo >> binLog2Dim;
+                            ijk[2] = pageIndexModulo - (ijk[1] << binLog2Dim);
+
+                            ijk = (ijk << bucketLog2Dim) + segmentCoord;
+                        }
+                    }
+                );
+            }
+        }
+    );
+
+    // move segment data
+
+    tbb::parallel_for(segmentRange,
+        MoveSegmentDataOp<PointIndexType>(indexArray, offsetSegments.get()));
 }
 
 
@@ -1000,16 +1006,10 @@ PointPartitioner<PointIndexType, BucketLog2Dim>::construct(
     mUsingCellCenteredTransform = cellCenteredTransform;
 
     point_partitioner_internal::partition(points, xform, BucketLog2Dim,
-        mPointIndices, mPageOffsets, mPageCount, mVoxelOffsets,
+        mPointIndices, mPageOffsets, mPageCoordinates, mPageCount, mVoxelOffsets,
             (voxelOrder || recordVoxelOffsets), cellCenteredTransform);
 
     const tbb::blocked_range<size_t> pageRange(0, mPageCount);
-    mPageCoordinates.reset(new Coord[mPageCount]);
-
-    tbb::parallel_for(pageRange,
-        point_partitioner_internal::LeafNodeOriginOp<PointArray, IndexType>
-            (mPageCoordinates, mPointIndices, mPageOffsets, points, xform,
-                BucketLog2Dim, cellCenteredTransform));
 
     if (mVoxelOffsets && voxelOrder) {
         tbb::parallel_for(pageRange, point_partitioner_internal::VoxelOrderOp<
