@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 #ifndef OPENVDB_TREE_LEAFBUFFER_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_LEAFBUFFER_HAS_BEEN_INCLUDED
@@ -66,30 +39,9 @@ namespace internal {
 template<typename T>
 struct LeafBufferFlags
 {
-#if OPENVDB_ABI_VERSION_NUMBER >= 5
     /// The type of LeafBuffer::mOutOfCore
     using type = tbb::atomic<Index32>;
     static constexpr bool IsAtomic = true;
-#else // OPENVDB_ABI_VERSION_NUMBER < 5
-    // These structs need to have the same data members as LeafBuffer.
-    struct Atomic { union { T* data; void* ptr; }; tbb::atomic<Index32> i; tbb::spin_mutex mutex; };
-    struct NonAtomic { union { T* data; void* ptr; }; Index32 i; tbb::spin_mutex mutex; };
-
-#ifndef __INTEL_COMPILER
-    /// @c true if LeafBuffer::mOutOfCore is atomic, @c false otherwise
-    static constexpr bool IsAtomic = ((sizeof(Atomic) == sizeof(NonAtomic))
-         && (offsetof(Atomic, i) == offsetof(NonAtomic, i)));
-#else
-    // We can't use offsetof() with ICC, because it requires the arguments
-    // to be POD types.  (C++11 requires only that they be standard layout types,
-    // which Atomic and NonAtomic are.)
-    static constexpr bool IsAtomic = (sizeof(Atomic) == sizeof(NonAtomic));
-#endif
-    /// The size of a LeafBuffer when LeafBuffer::mOutOfCore is atomic
-    static constexpr size_t size = sizeof(Atomic);
-    /// The type of LeafBuffer::mOutOfCore
-    using type = typename std::conditional<IsAtomic, tbb::atomic<Index32>, Index32>::type;
-#endif
 };
 
 } // namespace internal
@@ -106,7 +58,6 @@ public:
     using NodeMaskType = util::NodeMask<Log2Dim>;
     static const Index SIZE = 1 << 3 * Log2Dim;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
     struct FileInfo
     {
         FileInfo(): bufpos(0) , maskpos(0) {}
@@ -115,23 +66,7 @@ public:
         io::MappedFile::Ptr mapping;
         SharedPtr<io::StreamMetadata> meta;
     };
-#endif
 
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    /// Default constructor
-    LeafBuffer(): mData(new ValueType[SIZE]) {}
-    /// Construct a buffer populated with the specified value.
-    explicit LeafBuffer(const ValueType& val): mData(new ValueType[SIZE]) { this->fill(val); }
-    /// Copy constructor
-    LeafBuffer(const LeafBuffer& other): mData(new ValueType[SIZE]) { *this = other; }
-    /// Destructor
-    ~LeafBuffer() { delete[] mData; }
-
-    /// Return @c true if this buffer's values have not yet been read from disk.
-    bool isOutOfCore() const { return false; }
-    /// Return @c true if memory for this buffer has not yet been allocated.
-    bool empty() const { return (mData == nullptr); }
-#else
     /// Default constructor
     inline LeafBuffer(): mData(new ValueType[SIZE]) { mOutOfCore = 0; }
     /// Construct a buffer populated with the specified value.
@@ -147,7 +82,6 @@ public:
     bool isOutOfCore() const { return bool(mOutOfCore); }
     /// Return @c true if memory for this buffer has not yet been allocated.
     bool empty() const { return !mData || this->isOutOfCore(); }
-#endif
     /// Allocate memory for this buffer if it has not already been allocated.
     bool allocate() { if (mData == nullptr) mData = new ValueType[SIZE]; return true; }
 
@@ -201,24 +135,13 @@ private:
 
     bool deallocate();
 
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    void setOutOfCore(bool) {}
-    void loadValues() const {}
-    void doLoad() const {}
-    bool detachFromFile() { return false; }
-#else
     inline void setOutOfCore(bool b) { mOutOfCore = b; }
     // To facilitate inlining in the common case in which the buffer is in-core,
     // the loading logic is split into a separate function, doLoad().
     inline void loadValues() const { if (this->isOutOfCore()) this->doLoad(); }
     inline void doLoad() const;
     inline bool detachFromFile();
-#endif
 
-
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    ValueType* mData;
-#else
     using FlagsType = typename internal::LeafBufferFlags<ValueType>::type;
 
     union {
@@ -230,7 +153,6 @@ private:
     //int8_t mReserved[3]; // padding for alignment
 
     static const ValueType sZero;
-#endif
 
     friend class ::TestLeaf;
     // Allow the parent LeafNode to access this buffer's data pointer.
@@ -241,8 +163,6 @@ private:
 ////////////////////////////////////////
 
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
-
 template<typename T, Index Log2Dim>
 const T LeafBuffer<T, Log2Dim>::sZero = zeroVal<T>();
 
@@ -286,20 +206,14 @@ LeafBuffer<T, Log2Dim>::LeafBuffer(const LeafBuffer& other)
     }
 }
 
-#endif // OPENVDB_ABI_VERSION_NUMBER >= 3
-
 
 template<typename T, Index Log2Dim>
 inline void
 LeafBuffer<T, Log2Dim>::setValue(Index i, const ValueType& val)
 {
     assert(i < SIZE);
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    mData[i] = val;
-#else
     this->loadValues();
     if (mData) mData[i] = val;
-#endif
 }
 
 
@@ -308,15 +222,6 @@ inline LeafBuffer<T, Log2Dim>&
 LeafBuffer<T, Log2Dim>::operator=(const LeafBuffer& other)
 {
     if (&other != this) {
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-        if (other.mData != nullptr) {
-            this->allocate();
-            ValueType* target = mData;
-            const ValueType* source = other.mData;
-            Index n = SIZE;
-            while (n--) *target++ = *source++;
-        }
-#else // OPENVDB_ABI_VERSION_NUMBER >= 3
         if (this->isOutOfCore()) {
             this->detachFromFile();
         } else {
@@ -332,7 +237,6 @@ LeafBuffer<T, Log2Dim>::operator=(const LeafBuffer& other)
             Index n = SIZE;
             while (n--) *target++ = *source++;
         }
-#endif
     }
     return *this;
 }
@@ -371,9 +275,7 @@ inline void
 LeafBuffer<T, Log2Dim>::swap(LeafBuffer& other)
 {
     std::swap(mData, other.mData);
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
     std::swap(mOutOfCore, other.mOutOfCore);
-#endif
 }
 
 
@@ -382,12 +284,8 @@ inline Index
 LeafBuffer<T, Log2Dim>::memUsage() const
 {
     size_t n = sizeof(*this);
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    if (mData) n += SIZE * sizeof(ValueType);
-#else
     if (this->isOutOfCore()) n += sizeof(FileInfo);
     else if (mData) n += SIZE * sizeof(ValueType);
-#endif
     return static_cast<Index>(n);
 }
 
@@ -396,7 +294,6 @@ template<typename T, Index Log2Dim>
 inline const typename LeafBuffer<T, Log2Dim>::ValueType*
 LeafBuffer<T, Log2Dim>::data() const
 {
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
     this->loadValues();
     if (mData == nullptr) {
         LeafBuffer* self = const_cast<LeafBuffer*>(this);
@@ -404,7 +301,6 @@ LeafBuffer<T, Log2Dim>::data() const
         tbb::spin_mutex::scoped_lock lock(self->mMutex);
         if (mData == nullptr) self->mData = new ValueType[SIZE];
     }
-#endif
     return mData;
 }
 
@@ -412,14 +308,12 @@ template<typename T, Index Log2Dim>
 inline typename LeafBuffer<T, Log2Dim>::ValueType*
 LeafBuffer<T, Log2Dim>::data()
 {
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
     this->loadValues();
     if (mData == nullptr) {
         // This lock will be contended at most once.
         tbb::spin_mutex::scoped_lock lock(mMutex);
         if (mData == nullptr) mData = new ValueType[SIZE];
     }
-#endif
     return mData;
 }
 
@@ -429,14 +323,10 @@ inline const typename LeafBuffer<T, Log2Dim>::ValueType&
 LeafBuffer<T, Log2Dim>::at(Index i) const
 {
     assert(i < SIZE);
-#if OPENVDB_ABI_VERSION_NUMBER <= 2
-    return mData[i];
-#else
     this->loadValues();
     // We can't use the ternary operator here, otherwise Visual C++ returns
     // a reference to a temporary.
     if (mData) return mData[i]; else return sZero;
-#endif
 }
 
 
@@ -453,8 +343,6 @@ LeafBuffer<T, Log2Dim>::deallocate()
 }
 
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 3
-
 template<typename T, Index Log2Dim>
 inline void
 LeafBuffer<T, Log2Dim>::doLoad() const
@@ -506,8 +394,6 @@ LeafBuffer<T, Log2Dim>::detachFromFile()
     return false;
 }
 
-#endif // OPENVDB_ABI_VERSION_NUMBER >= 3
-
 
 ////////////////////////////////////////
 
@@ -583,7 +469,3 @@ template<Index Log2Dim> const bool LeafBuffer<bool, Log2Dim>::sOff = false;
 } // namespace openvdb
 
 #endif // OPENVDB_TREE_LEAFBUFFER_HAS_BEEN_INCLUDED
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
