@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 #include <cppunit/extensions/HelperMacros.h>
 #include <openvdb/Exceptions.h>
@@ -50,6 +23,7 @@ public:
     CPPUNIT_TEST(testIsConstant);
     CPPUNIT_TEST(testMedian);
     CPPUNIT_TEST(testFill);
+    CPPUNIT_TEST(testCount);
     CPPUNIT_TEST_SUITE_END();
 
     void testBuffer();
@@ -65,6 +39,7 @@ public:
     void testIsConstant();
     void testMedian();
     void testFill();
+    void testCount();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestLeaf);
@@ -561,6 +536,25 @@ TestLeaf::testFill()
     CPPUNIT_ASSERT_EQUAL(int(clippedBBox.volume()), int(leaf.onVoxelCount()));
 }
 
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
+void
+TestLeaf::testCount()
+{
+    using namespace openvdb;
+    const Coord origin(-9, -2, -8);
+    tree::LeafNode<float, 3> leaf(origin, 1.0f, false);
+
+    CPPUNIT_ASSERT_EQUAL(Index(3), leaf.log2dim());
+    CPPUNIT_ASSERT_EQUAL(Index(8), leaf.dim());
+    CPPUNIT_ASSERT_EQUAL(Index(512), leaf.size());
+    CPPUNIT_ASSERT_EQUAL(Index(512), leaf.numValues());
+    CPPUNIT_ASSERT_EQUAL(Index(0), leaf.getLevel());
+    CPPUNIT_ASSERT_EQUAL(Index(1), leaf.getChildDim());
+    CPPUNIT_ASSERT_EQUAL(Index(1), leaf.leafCount());
+    CPPUNIT_ASSERT_EQUAL(Index(0), leaf.nonLeafCount());
+    CPPUNIT_ASSERT_EQUAL(Index(0), leaf.childCount());
+
+    std::vector<Index> dims;
+    leaf.getNodeLog2Dims(dims);
+    CPPUNIT_ASSERT_EQUAL(size_t(1), dims.size());
+    CPPUNIT_ASSERT_EQUAL(Index(3), dims[0]);
+}
