@@ -24,11 +24,7 @@ namespace util {
 template<typename GridType, typename OpType, bool IsConst/*=false*/>
 struct GridProcessor {
     static inline void call(OpType& op, openvdb::GridBase::Ptr grid) {
-#ifdef _MSC_VER
-        op.operator()<GridType>(openvdb::gridPtrCast<GridType>(grid));
-#else
         op.template operator()<GridType>(openvdb::gridPtrCast<GridType>(grid));
-#endif
     }
 };
 
@@ -36,11 +32,7 @@ struct GridProcessor {
 template<typename GridType, typename OpType>
 struct GridProcessor<GridType, OpType, /*IsConst=*/true> {
     static inline void call(OpType& op, openvdb::GridBase::ConstPtr grid) {
-#ifdef _MSC_VER
-        op.operator()<GridType>(openvdb::gridConstPtrCast<GridType>(grid));
-#else
         op.template operator()<GridType>(openvdb::gridConstPtrCast<GridType>(grid));
-#endif
     }
 };
 
@@ -172,7 +164,7 @@ template <class TreeType>
 MinMaxVoxel<TreeType>::MinMaxVoxel(LeafArray& leafs)
     : mLeafArray(leafs)
     , mMin(std::numeric_limits<ValueType>::max())
-    , mMax(-mMin)
+    , mMax(std::numeric_limits<ValueType>::lowest())
 {
 }
 
@@ -182,7 +174,7 @@ inline
 MinMaxVoxel<TreeType>::MinMaxVoxel(const MinMaxVoxel<TreeType>& rhs, tbb::split)
     : mLeafArray(rhs.mLeafArray)
     , mMin(std::numeric_limits<ValueType>::max())
-    , mMax(-mMin)
+    , mMax(std::numeric_limits<ValueType>::lowest())
 {
 }
 
