@@ -132,8 +132,9 @@ GU_PrimVDB::buildFromGridAdapter(GU_Detail& gdp, void* gridPtr,
             GU_PrimVDB::createMetadataFromGridAttrs(*grid, *vdb, gdp);
 
             // Copy the source's visualization options.
-            GEO_VolumeOptions   visopt = src->getVisOptions();
-            vdb->setVisualization(visopt.myMode, visopt.myIso, visopt.myDensity);
+            GEO_VolumeOptions visopt = src->getVisOptions();
+            vdb->setVisualization(visopt.myMode, visopt.myIso, visopt.myDensity,
+                                  visopt.myLod);
         }
 
         // Ensure that certain metadata exists (grid name, grid class, etc.).
@@ -168,12 +169,14 @@ GU_PrimVDB::buildFromGridAdapter(GU_Detail& gdp, void* gridPtr,
             if (grid->getGridClass() == openvdb::GRID_LEVEL_SET)
             {
                 vdb->setVisualization(GEO_VOLUMEVIS_ISO,
-                                      vdb->getVisIso(), vdb->getVisDensity());
+                                      vdb->getVisIso(), vdb->getVisDensity(),
+                                      vdb->getVisLod());
             }
             else
             {
                 vdb->setVisualization(GEO_VOLUMEVIS_SMOKE,
-                                      vdb->getVisIso(), vdb->getVisDensity());
+                                      vdb->getVisIso(), vdb->getVisDensity(),
+                                      vdb->getVisLod());
             }
         }
     }
@@ -427,7 +430,8 @@ GU_PrimVDB::buildFromPrimVolume(
     vol.getRes(rx, ry, rz);
     prim_vdb->setSpaceTransform(vol.getSpaceTransform(), UT_Vector3R(rx,ry,rz));
     prim_vdb->setVisualization(
-                vol.getVisualization(), vol.getVisIso(), vol.getVisDensity());
+                vol.getVisualization(), vol.getVisIso(), vol.getVisDensity(),
+                GEO_VOLUMEVISLOD_FULL);
     return prim_vdb;
 }
 
@@ -527,8 +531,9 @@ GU_PrimVDB::registerMyself(GA_PrimitiveFactory *factory)
 #endif
 
     if (!theDefinition) {
+        std::cerr << "WARNING: Unable to register custom GU_PrimVDB\n";
         if (!factory->lookupDefinition("VDB")) {
-            //std::cerr << "WARNING: failed to register GU_PrimVDB\n";
+            std::cerr << "WARNING: failed to register GU_PrimVDB\n";
         }
         return;
     }
@@ -1336,6 +1341,28 @@ VoxelArrayVolume<TUPLE_SIZE>::copyToTile(
             }
         }
     }
+
+    // Enable this to do slow code path verification
+#if 0
+    for (int tuple_i = 0; tuple_i < TUPLE_SIZE; ++tuple_i) {
+        VoxelTileF* tile = tiles[tuple_i];
+        fpreal32* data = tile->rawData();
+        Coord xyz;
+        for (xyz[2] = 0; xyz[2] < tile_res[2]; ++xyz[2]) {
+            for (xyz[1] = 0; xyz[1] < tile_res[1]; ++xyz[1]) {
+                for (xyz[0] = 0; xyz[0] < tile_res[0]; ++xyz[0]) {
+                    Coord ijk = beg + xyz;
+                    if (!compareVoxel(xyz, tile, data,
+                                      acc.getValue(ijk), tuple_i)) {
+                        UT_ASSERT(!"Voxels are different");
+                        compareVoxel(xyz, tile, data,
+                                     acc.getValue(ijk), tuple_i);
+                    }
+                }
+            }
+        }
+    }
+#endif
 }
 
 template<typename TreeType, typename VolumeT, bool aligned>
@@ -2177,6 +2204,7 @@ GU_PrimVDB::createMetadataFromAttrsAdapter(
         }
 
         case GA_STORECLASS_INVALID: break;
+        case GA_STORECLASS_DICT: break;
         case GA_STORECLASS_OTHER: break;
         }
     }
