@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @file PointUtils.cc
 /// @authors Dan Bailey, Nick Avramoussis, Richard Kwok
@@ -49,7 +22,6 @@
 #include <CH/CH_Manager.h> // for CHgetEvalTime
 #include <PRM/PRM_SpareData.h>
 #include <SOP/SOP_Node.h>
-#include <UT/UT_Version.h>
 #include <UT/UT_UniquePtr.h>
 
 #include <algorithm>
@@ -498,14 +470,15 @@ convertAttributeFromHoudini(PointDataTree& tree, const tools::PointIndexTree& in
     ValueType value = hvdb::evalAttrDefault<ValueType>(defaults, 0);
 
     // empty metadata if default is zero
-    Metadata::Ptr defaultValue;
     if (!math::isZero<ValueType>(value)) {
-        defaultValue = TypedMetadata<ValueType>(value).copy();
+        TypedMetadata<ValueType> defaultValue(value);
+        appendAttribute<ValueType, CodecType>(tree, name, zeroVal<ValueType>(),
+            stride, /*constantstride=*/true, &defaultValue);
+    } else {
+        appendAttribute<ValueType, CodecType>(tree, name, zeroVal<ValueType>(),
+            stride, /*constantstride=*/true);
     }
 
-    appendAttribute<ValueType, CodecType>(tree, name, zeroVal<ValueType>(),
-        stride, /*constantstride=*/true, defaultValue);
-
     HoudiniAttribute houdiniAttribute(*attribute);
     populateAttribute<PointDataTree, tools::PointIndexTree, HoudiniAttribute>(
         tree, indexTree, name, houdiniAttribute, stride);
@@ -1012,20 +985,24 @@ convertHoudiniToPointDataGrid(const GU_Detail& ptGeo,
         std::vector<short> inGroup(groupVectorSize, short(0));
 
         // Set group membership in tree
-        // @todo parallelize group membership construction
 
         for (auto it = elementGroups.beginTraverse(), itEnd = elementGroups.endTraverse();
             it != itEnd; ++it) {
 
-            GA_Offset start, end;
-            GA_Range range(**it);
-            for (GA_Iterator rangeIt = range.begin(); rangeIt.blockAdvance(start, end); ) {
-                end = std::min(end, numHoudiniPoints);
-                for (GA_Offset off = start; off < end; ++off) {
-                    UT_ASSERT(off < GA_Offset(numHoudiniPoints));
-                    inGroup[off] = short(1);
+            const GA_Range range(**it);
+            tbb::parallel_for(GA_SplittableRange(range),
+                [&ptGeo, &inGroup](const GA_SplittableRange& r) {
+                for (GA_PageIterator pit = r.beginPages(); !pit.atEnd(); ++pit) {
+                    GA_Offset start, end;
+                    for (GA_Iterator iter = pit.begin(); iter.blockAdvance(start, end);) {
+                        for (GA_Offset off = start; off < end; ++off) {
+                            const GA_Index idx = ptGeo.pointIndex(off);
+                            UT_ASSERT(idx < GA_Index(inGroup.size()));
+                            inGroup[idx] = short(1);
+                        }
+                    }
                 }
-            }
+            });
 
             const Name groupName = (*it)->getName().toStdString();
             setGroup(tree, indexTree, inGroup, groupName);
@@ -1858,7 +1835,3 @@ const PRM_ChoiceList VDBPointsGroupMenu(PRM_CHOICELIST_TOGGLE,
 #endif
 
 } // namespace openvdb_houdini
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
