@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file SOP_OpenVDB_Advect_Points.cc
 ///
@@ -1025,7 +998,7 @@ SOP_OpenVDB_Advect_Points::Cache::cookVDBSop(OP_Context& context)
                     {
                         VDBPointsAdvection<openvdb::points::PointDataGrid> advection(
                             outputGrid, parms, boss);
-                        GEOvdbProcessTypedGridVec3(*parms.mVelPrim, advection);
+                        hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*parms.mVelPrim, advection);
                         break;
                     }
                     case PROPAGATION_TYPE_PROJECTION: break; // not implemented
@@ -1040,13 +1013,13 @@ SOP_OpenVDB_Advect_Points::Cache::cookVDBSop(OP_Context& context)
             case PROPAGATION_TYPE_CONSTRAINED_ADVECTION:
             {
                 Advection advection(parms, boss);
-                GEOvdbProcessTypedGridVec3(*parms.mVelPrim, advection);
+                hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*parms.mVelPrim, advection);
                 break;
             }
             case PROPAGATION_TYPE_PROJECTION:
             {
                 Projection projection(parms, boss);
-                GEOvdbProcessTypedGridVec3(*parms.mCptPrim, projection);
+                hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*parms.mVelPrim, projection);
                 break;
             }
             case PROPAGATION_TYPE_UNKNOWN: break;
@@ -1176,7 +1149,3 @@ SOP_OpenVDB_Advect_Points::Cache::evalAdvectionParms(
 
     return true;
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
