@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file SOP_OpenVDB_Analysis.cc
 ///
@@ -44,8 +17,6 @@
 #include <openvdb/tools/GridTransformer.h>
 
 #include <UT/UT_Interrupt.h>
-#include <UT/UT_Version.h>
-#include <GEO/GEO_PrimVDB.h> // for GEOvdbProcessTypedGridScalar(), etc.
 
 #include <sstream>
 #include <stdexcept>
@@ -399,8 +370,9 @@ SOP_OpenVDB_Analysis::Cache::cookVDBSop(OP_Context& context)
             hvdb::VdbPrimCIterator maskIt(maskGeo, maskGroup);
             if (maskIt) {
                 MaskOp op;
-                UTvdbProcessTypedGridTopology(maskIt->getStorageType(), maskIt->getGrid(), op);
-                maskGrid = op.mMaskGrid;
+                if (hvdb::GEOvdbApply<hvdb::AllGridTypes>(**maskIt, op)) {
+                    maskGrid = op.mMaskGrid;
+                }
             }
 
             if (!maskGrid) addWarning(SOP_MESSAGE, "Mask VDB not found.");
@@ -421,64 +393,72 @@ SOP_OpenVDB_Analysis::Cache::cookVDBSop(OP_Context& context)
                 case OP_GRADIENT: // gradient of scalar field
                 {
                     ToolOp<cvdb::tools::Gradient> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridScalar(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::NumericGridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_gradient";
                     break;
                 }
                 case OP_CURVATURE: // mean curvature of scalar field
                 {
                     ToolOp<cvdb::tools::MeanCurvature> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridScalar(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::NumericGridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_curvature";
                     break;
                 }
                 case OP_LAPLACIAN: // Laplacian of scalar field
                 {
                     ToolOp<cvdb::tools::Laplacian> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridScalar(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::NumericGridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_laplacian";
                     break;
                 }
                 case OP_CPT: // closest point transform of scalar level set
                 {
                     ToolOp<cvdb::tools::Cpt> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridScalar(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::NumericGridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_cpt";
                     break;
                 }
                 case OP_DIVERGENCE: // divergence of vector field
                 {
                     ToolOp<cvdb::tools::Divergence> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridVec3(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_divergence";
                     break;
                 }
                 case OP_CURL: // curl (rotation) of vector field
                 {
                     ToolOp<cvdb::tools::Curl> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridVec3(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_curl";
                     break;
                 }
                 case OP_MAGNITUDE: // magnitude of vector field
                 {
                     ToolOp<cvdb::tools::Magnitude> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridVec3(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_magnitude";
                     break;
                 }
                 case OP_NORMALIZE: // normalize vector field
                 {
                     ToolOp<cvdb::tools::Normalize> op(threaded, boss, maskGrid.get());
-                    ok = GEOvdbProcessTypedGridVec3(*vdb, op, /*makeUnique=*/false);
-                    if (ok) outGrid = op.mOutGrid;
+                    if (hvdb::GEOvdbApply<hvdb::Vec3GridTypes>(*vdb, op, /*makeUnique=*/false)) {
+                        outGrid = op.mOutGrid;
+                    }
                     operationName = "_normalize";
                     break;
                 }
@@ -489,7 +469,7 @@ SOP_OpenVDB_Analysis::Cache::cookVDBSop(OP_Context& context)
                 std::ostringstream ss;
                 ss << "Can't compute " << sOpName[whichOp] << " from grid";
                 if (inGridName.isstring()) ss << " " << inGridName;
-                ss << " of type " << vdb->getGrid().valueType();
+                ss << " of type " << UTvdbGetGridTypeString(vdb->getGrid());
                 addWarning(SOP_MESSAGE, ss.str().c_str());
             }
 
@@ -512,7 +492,3 @@ SOP_OpenVDB_Analysis::Cache::cookVDBSop(OP_Context& context)
     }
     return error();
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
