@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @file SOP_OpenVDB_Clip.cc
 ///
@@ -438,8 +411,9 @@ struct MaskClipOp
         if (mask) {
             // Dispatch on the mask grid type, now that the source grid type is resolved.
             MaskClipDispatchOp<GridType> op(grid, inside);
-            UTvdbProcessTypedGridTopology(UTvdbGetGridType(*mask), *mask, op);
-            outputGrid = op.outputGrid;
+            if (mask->apply<hvdb::AllGridTypes>(op)) {
+                outputGrid = op.outputGrid;
+            }
         }
     }
 
@@ -575,7 +549,7 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
                     if (maskIt->getConstGrid().getGridClass() == openvdb::GRID_LEVEL_SET) {
                         // If the mask grid is a level set, extract an interior mask from it.
                         LevelSetMaskOp op;
-                        GEOvdbProcessTypedGridScalar(**maskIt, op);
+                        hvdb::GEOvdbApply<hvdb::NumericGridTypes>(**maskIt, op);
                         maskGrid = op.outputGrid;
                     } else {
                         maskGrid = maskIt->getConstGridPtr();
@@ -595,17 +569,14 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
                             "nonuniform padding is not supported for mask clipping");
                     }
                     if (const int dilation = int(std::round(paddingInVoxels[0]))) {
-                        const auto maskType = UTvdbGetGridType(*maskGrid);
                         DilatedMaskOp op{dilation};
-                        if (!UTvdbProcessTypedGridTopology(maskType, *maskGrid, op)) {
-                            UTvdbProcessTypedGridPoint(maskType, *maskGrid, op);
-                        }
+                        maskGrid->apply<hvdb::AllGridTypes>(op);
                         if (op.maskGrid) maskGrid = op.maskGrid;
                     }
                 }
             } else {
                 UT_BoundingBox box;
-                maskGeo->computeQuickBounds(box);
+                maskGeo->getBBox(&box);
 
                 clipBox.min()[0] = box.xmin();
                 clipBox.min()[1] = box.ymin();
@@ -643,7 +614,7 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
 
             if (maskGrid) {
                 MaskClipOp op{maskGrid, inside};
-                if (GEOvdbProcessTypedGridTopology(**it, op)) { // all Houdini-supported grid types
+                if (hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(**it, op)) { // all Houdini-supported volume grid types
                     outGrid = op.outputGrid;
                 } else if (inGrid.isType<openvdb::points::PointDataGrid>()) {
                     addWarning(SOP_MESSAGE,
@@ -651,7 +622,7 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
                 }
             } else if (useCamera) {
                 FrustumClipOp op{mFrustum, inside};
-                if (GEOvdbProcessTypedGridTopology(**it, op)) { // all Houdini-supported grid types
+                if (hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(**it, op)) { // all Houdini-supported volume grid types
                     outGrid = op.outputGrid;
                 } else if (inGrid.isType<openvdb::points::PointDataGrid>()) {
                     addWarning(SOP_MESSAGE,
@@ -659,7 +630,7 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
                 }
             } else {
                 BBoxClipOp op{clipBox, inside};
-                if (GEOvdbProcessTypedGridTopology(**it, op)) { // all Houdini-supported grid types
+                if (hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(**it, op)) { // all Houdini-supported volume grid types
                     outGrid = op.outputGrid;
                 } else if (inGrid.isType<openvdb::points::PointDataGrid>()) {
                     if (inside) {
@@ -693,7 +664,3 @@ SOP_OpenVDB_Clip::Cache::cookVDBSop(OP_Context& context)
 
     return error();
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
