@@ -26,7 +26,6 @@
 #include <GU/GU_PrimPolySoup.h>
 #include <SYS/SYS_Math.h>
 #include <UT/UT_Interrupt.h>
-#include <UT/UT_Version.h>
 #include <UT/UT_VoxelArray.h>
 #include <UT/UT_UniquePtr.h>
 
@@ -795,7 +794,7 @@ getVDBTypeFromNameAndPrecision(const UT_String& name, int bits)
 ////////////////////////////////////////
 
 
-// Functor for use with GEOvdbProcessTypedGrid*() to create a copy of a grid,
+// Functor for use with GEOvdbApply() to create a copy of a grid,
 // but with a new value type
 struct GridCopyOp
 {
@@ -894,7 +893,7 @@ getMaskFromGrid(const hvdb::GridCPtr& gridPtr, double isovalue = 0.0)
             maskGridPtr = gridPtr;
         } else {
             InteriorMaskOp op{isovalue};
-            UTvdbProcessTypedGridTopology(UTvdbGetGridType(*gridPtr), *gridPtr, op);
+            gridPtr->apply<hvdb::VolumeGridTypes>(op);
             maskGridPtr = op.outGridPtr;
         }
     }
@@ -1034,7 +1033,7 @@ SOP_OpenVDB_Convert::Cache::convertVDBType(
             op.outType = outType;
             // Create a copy of the grid, but with a different value type.
             // Store the copy as op.outGrid.
-            GEOvdbProcessTypedGridTopology(*it.getPrimitive(), op);
+            hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(**it, op);
             if (op.outGrid) {
                 auto& grid = *op.outGrid;
                 grid.removeMeta("value_type");
@@ -1414,7 +1413,7 @@ SOP_OpenVDB_Convert::Cache::convertToPoly(
         for (; vdbIt; ++vdbIt) {
 
             if (boss.wasInterrupted()) break;
-            GEOvdbProcessTypedGridScalar(*vdbIt.getPrimitive(), mesher);
+            hvdb::GEOvdbApply<hvdb::NumericGridTypes>(**vdbIt, mesher);
 
             delGroup->addOffset(vdbIt.getOffset());
 
