@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file SOP_OpenVDB_Diagnostics.cc
 ///
@@ -43,19 +16,10 @@
 #include <openvdb/tools/LevelSetRebuild.h>
 #include <openvdb/tools/LevelSetTracker.h> // LevelSetTracker::normalize
 
-#include <UT/UT_Version.h>
 #include <UT/UT_Interrupt.h>
 #include <UT/UT_UniquePtr.h>
 #include <PRM/PRM_Parm.h>
 
-#ifdef SESI_OPENVDB
-  #include <hboost/mpl/at.hpp>
-  namespace boostmpl = hboost::mpl;
-#else
-  #include <boost/mpl/at.hpp>
-  namespace boostmpl = boost::mpl;
-#endif
-
 #include <memory>
 #include <string>
 #include <sstream>
@@ -229,7 +193,7 @@ struct Visitor
     using LeafNodeType = typename TreeType::LeafNodeType;
     using RootNodeType = typename TreeType::RootNodeType;
     using NodeChainType = typename RootNodeType::NodeChainType;
-    using InternalNodeType = typename boostmpl::at<NodeChainType, boostmpl::int_<1>>::type;
+    using InternalNodeType = typename NodeChainType::template Get<1>;
 
     using BoolTreeType = typename TreeType::template ValueConverter<bool>::Type;
     using BoolTreePtr = typename BoolTreeType::Ptr;
@@ -1975,7 +1939,7 @@ SOP_OpenVDB_Diagnostics::Cache::cookVDBSop(OP_Context& context)
             tests.setPrimitiveName(it.getPrimitiveName().toStdString());
             tests.setPrimitiveIndex(int(it.getIndex()));
 
-            GEOvdbProcessTypedGridTopology(**it, tests, /*makeUnique=*/false);
+            hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(**it, tests, /*makeUnique=*/false);
 
             if (tests.replacementGrid()) {
                 hvdb::replaceVdbPrimitive(*gdp, tests.replacementGrid(), **it, true,
@@ -1994,7 +1958,3 @@ SOP_OpenVDB_Diagnostics::Cache::cookVDBSop(OP_Context& context)
     }
     return error();
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
