@@ -12,7 +12,6 @@
 #include <PRM/PRM_Parm.h>
 #include <UT/UT_Interrupt.h>
 #include <UT/UT_UniquePtr.h>
-#include <UT/UT_Version.h>
 
 #include <memory>
 #include <stdexcept>
@@ -365,7 +364,7 @@ SOP_OpenVDB_Fill::Cache::cookVDBSop(OP_Context& context)
                 openvdb::BBoxd bbox;
                 if (const GU_Detail* refGeo = inputGeo(1)) {
                     UT_BoundingBox b;
-                    refGeo->computeQuickBounds(b);
+                    refGeo->getBBox(&b);
                     if (!b.isValid()) {
                         throw std::runtime_error("no reference geometry found");
                     }
