@@ -22,7 +22,6 @@
 #include <CH/CH_Manager.h>
 #include <PRM/PRM_Parm.h>
 #include <PRM/PRM_SharedFunc.h>
-#include <UT/UT_Version.h>
 
 #include <algorithm> // for std::max()
 #include <sstream>
@@ -46,7 +45,7 @@ evalAttrType(const UT_String& attrStr, UT_String& attrName, int& attrClass)
 {
     std::string str = attrStr.toStdString();
 
-    const size_t idx = str.find_first_of(".");
+    const size_t idx = str.find_first_of('.');
     if (idx == std::string::npos) return false;
 
     attrName = str.substr(idx + 1, str.size() - 1);
@@ -76,25 +75,26 @@ sopBuildAttrMenu(void* data, PRM_Name* menuEntries, int themenusize,
 {
     if (data == nullptr || menuEntries == nullptr || spare == nullptr) return;
 
+    size_t menuIdx = 0;
+
+    menuEntries[menuIdx].setToken("point.v");
+    menuEntries[menuIdx++].setLabel("point.v");
+
     SOP_Node* sop = CAST_SOPNODE(static_cast<OP_Node*>(data));
 
     if (sop == nullptr) {
         // terminate and quit
-        menuEntries[0].setToken(0);
-        menuEntries[0].setLabel(0);
+        menuEntries[menuIdx].setToken(0);
+        menuEntries[menuIdx].setLabel(0);
         return;
     }
 
     int inputIndex = lookupAttrInput(spare);
     const GU_Detail* gdp = sop->getInputLastGeo(inputIndex, CHgetEvalTime());
-    size_t menuIdx = 0, menuEnd(themenusize - 2);
+    size_t menuEnd(themenusize - 2);
 
     if (gdp) {
 
-        // null object
-        menuEntries[menuIdx].setToken("");
-        menuEntries[menuIdx++].setLabel("- no attribute selected -");
-
         // point attribute names
         GA_AttributeDict::iterator iter = gdp->pointAttribs().begin(GA_SCOPE_PUBLIC);
 
@@ -169,7 +169,7 @@ sopBuildAttrMenu(void* data, PRM_Name* menuEntries, int themenusize,
 }
 
 const PRM_ChoiceList PrimAttrMenu(
-    PRM_ChoiceListType(PRM_CHOICELIST_EXCLUSIVE | PRM_CHOICELIST_REPLACE), sopBuildAttrMenu);
+    PRM_ChoiceListType(PRM_CHOICELIST_REPLACE), sopBuildAttrMenu);
 
 } // unnamed namespace
 
@@ -685,11 +685,11 @@ SOP_OpenVDB_From_Polygons::updateParmsFlags()
     GA_ROAttributeRef attrRef;
     int attrClass = POINT_ATTR;
     const GU_Detail* meshGdp = this->getInputLastGeo(0, time);
-    if (meshGdp) {
-        for (int i = 1, N = static_cast<int>(evalInt("attrList", 0, time)); i <= N; ++i) {
-
+    for (int i = 1, N = static_cast<int>(evalInt("attrList", 0, time)); i <= N; ++i) {
+        bool isVector = true;
+        if (meshGdp) {
+            isVector = false;
             evalStringInst("attribute#", &i, attrStr, 0, time);
-            bool isVector = false;
 
             if (attrStr.length() != 0 && evalAttrType(attrStr, attrName, attrClass)) {
 
@@ -717,10 +717,10 @@ SOP_OpenVDB_From_Polygons::updateParmsFlags()
                     }
                 }
             }
-
-            changed |= enableParmInst("vecType#", &i, isVector);
-            changed |= setVisibleStateInst("vecType#", &i, isVector);
         }
+
+        changed |= enableParmInst("vectype#", &i, isVector);
+        changed |= setVisibleStateInst("vectype#", &i, isVector);
     }
     return changed;
 }
