@@ -12,7 +12,6 @@
 #include <openvdb/math/Math.h> // Tolerance and isApproxEqual
 #include <openvdb/tools/ValueTransformer.h>
 
-#include <UT/UT_Version.h>
 #include <UT/UT_Ramp.h>
 #include <GU/GU_Detail.h>
 #include <PRM/PRM_Parm.h>
@@ -21,14 +20,6 @@
 #include <tbb/parallel_for.h>
 #include <tbb/parallel_reduce.h>
 
-#ifdef SESI_OPENVDB
-  #include <hboost/mpl/at.hpp>
-  namespace boostmpl = hboost::mpl;
-#else
-  #include <boost/mpl/at.hpp>
-  namespace boostmpl = boost::mpl;
-#endif
-
 #include <algorithm>
 #include <cmath>
 #include <map>
@@ -178,7 +169,7 @@ evalMinMax(const TreeType& tree,
     { // eval first tiles
         using RootNodeType = typename TreeType::RootNodeType;
         using NodeChainType = typename RootNodeType::NodeChainType;
-        using InternalNodeType = typename boostmpl::at<NodeChainType, boostmpl::int_<1>>::type;
+        using InternalNodeType = typename NodeChainType::template Get<1>;
 
         std::vector<const InternalNodeType*> nodes;
         tree.getNodes(nodes);
@@ -221,7 +212,7 @@ deactivateBackgroundValues(TreeType& tree)
     { // eval first tiles
         using RootNodeType = typename TreeType::RootNodeType;
         using NodeChainType = typename RootNodeType::NodeChainType;
-        using InternalNodeType = typename boostmpl::at<NodeChainType, boostmpl::int_<1>>::type;
+        using InternalNodeType = typename NodeChainType::template Get<1>;
 
         std::vector<InternalNodeType*> nodes;
         tree.getNodes(nodes);
@@ -674,7 +665,7 @@ SOP_OpenVDB_Remap::Cache::cookVDBSop(OP_Context& context)
             remap.setPrimitiveName(it.getPrimitiveName().toStdString());
             remap.setPrimitiveIndex(int(it.getIndex()));
 
-            GEOvdbProcessTypedGrid(**it, remap, /*makeUnique=*/true);
+            hvdb::GEOvdbApply<hvdb::NumericGridTypes::Append<hvdb::Vec3GridTypes>>(**it, remap);
 
             GU_PrimVDB* vdbPrim = *it;
             const GEO_VolumeOptions& visOps = vdbPrim->getVisOptions();
