@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file SOP_OpenVDB_Remove_Divergence.cc
 ///
@@ -44,7 +17,6 @@
 
 #include <UT/UT_Interrupt.h>
 #include <UT/UT_StringArray.h>
-#include <UT/UT_Version.h>
 #include <GU/GU_Detail.h>
 #include <PRM/PRM_Parm.h>
 #include <GA/GA_Handle.h>
@@ -759,8 +731,7 @@ removeDivergenceWithColliderGrid(SolverParms& parms, const BoundaryOpType& bound
         if (parms.colliderType == CT_BBOX) {
             op.correctVelocity(BBoxConstAccessor(parms));
         } else {
-            UTvdbProcessTypedGridTopology(
-                UTvdbGetGridType(*parms.colliderGrid), *parms.colliderGrid, op);
+            parms.colliderGrid->apply<hvdb::VolumeGridTypes>(op);
         }
     }
 
@@ -825,8 +796,7 @@ processGrid(SolverParms& parms)
             // If a dynamic collider grid was supplied, its active values define
             // the velocities of solid obstacles.
             ColliderDispatchOp<VelocityGridType> op(parms);
-            success = UTvdbProcessTypedGridVec3(
-                UTvdbGetGridType(*parms.colliderGrid), *parms.colliderGrid, op);
+            success = parms.colliderGrid->apply<hvdb::Vec3GridTypes>(op);
             if (success) success = op.success;
             break;
         }
@@ -902,7 +872,7 @@ SOP_OpenVDB_Remove_Divergence::Cache::cookVDBSop(
             if (colliderTypeStr == "bbox") {
                 // Use the bounding box of the reference geometry as a collider.
                 UT_BoundingBox box;
-                colliderGeo->computeQuickBounds(box);
+                colliderGeo->getBBox(&box);
                 parms.colliderBBox.min() = openvdb::Vec3d(box.xmin(), box.ymin(), box.zmin());
                 parms.colliderBBox.max() = openvdb::Vec3d(box.xmax(), box.ymax(), box.zmax());
                 parms.colliderType = CT_BBOX;
@@ -918,7 +888,7 @@ SOP_OpenVDB_Remove_Divergence::Cache::cookVDBSop(
                     if (colliderIt->getConstGrid().getGridClass() == openvdb::GRID_LEVEL_SET) {
                         // If the collider grid is a level set, extract an interior mask from it.
                         LevelSetMaskOp op;
-                        if (GEOvdbProcessTypedGridScalar(**colliderIt, op)) {
+                        if (hvdb::GEOvdbApply<hvdb::NumericGridTypes>(**colliderIt, op)) {
                             parms.colliderGrid = op.outputGrid;
                         }
                     }
@@ -961,8 +931,7 @@ SOP_OpenVDB_Remove_Divergence::Cache::cookVDBSop(
                         parms.colliderType = CT_STATIC;
                         ColliderMaskOp op;
                         op.mask = ColliderMaskGrid::create();
-                        UTvdbProcessTypedGridTopology(UTvdbGetGridType(*parms.colliderGrid),
-                            *parms.colliderGrid, op);
+                        parms.colliderGrid->apply<hvdb::AllGridTypes>(op);
                         parms.colliderGrid = op.mask;
                     }
                 }
@@ -1052,7 +1021,3 @@ SOP_OpenVDB_Remove_Divergence::Cache::cookVDBSop(
 
     return error();
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
