@@ -14,14 +14,13 @@
 #include <houdini_utils/ParmFactory.h>
 #include <openvdb_houdini/Utils.h>
 #include <openvdb_houdini/UT_VDBTools.h> // for GridTransformOp, et al.
-#include <openvdb_houdini/UT_VDBUtils.h> // for UTvdbProcessTypedGridReal()
+#include <openvdb_houdini/UT_VDBUtils.h> // for UTvdbGridCast()
 #include <openvdb_houdini/SOP_NodeVDB.h>
 #include <openvdb/openvdb.h>
 #include <openvdb/tools/GridTransformer.h>
 #include <openvdb/tools/LevelSetRebuild.h>
 #include <openvdb/tools/VectorTransformer.h> // for transformVectors()
 #include <UT/UT_Interrupt.h>
-#include <UT/UT_Version.h>
 #include <functional>
 #include <stdexcept>
 #include <string>
@@ -348,7 +347,7 @@ SOP_OpenVDB_Resample::SOP_OpenVDB_Resample(OP_Network* net, const char* name, OP
 
 namespace {
 
-// Helper class for use with UTvdbProcessTypedGrid()
+// Helper class for use with GridBase::apply()
 struct RebuildOp
 {
     std::function<void (const std::string&)> addWarning;
@@ -376,7 +375,7 @@ struct RebuildOp
 }; // struct RebuildOp
 
 
-// Functor for use with UTvdbProcessTypedGridVec3() to apply a transform
+// Functor for use with GridBase::apply() to apply a transform
 // to the voxel values of vector-valued grids
 struct VecXformOp
 {
@@ -518,7 +517,7 @@ SOP_OpenVDB_Resample::Cache::cookVDBSop(OP_Context& context)
                     RebuildOp op;
                     op.addWarning = addWarningCB;
                     op.xform = *refXform;
-                    UTvdbProcessTypedGridReal(valueType, grid, op);
+                    grid.apply<hvdb::RealGridTypes>(op);
                     outGrid = op.outGrid;
 
                 } else {
@@ -529,13 +528,13 @@ SOP_OpenVDB_Resample::Cache::cookVDBSop(OP_Context& context)
 
                     if (curOrder == 0) {
                         hvdb::GridResampleToMatchOp<openvdb::tools::PointSampler> op(outGrid);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        grid.apply<hvdb::AllGridTypes>(op);
                     } else if (curOrder == 1) {
                         hvdb::GridResampleToMatchOp<openvdb::tools::BoxSampler> op(outGrid);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        grid.apply<hvdb::AllGridTypes>(op);
                     } else if (curOrder == 2) {
                         hvdb::GridResampleToMatchOp<openvdb::tools::QuadraticSampler> op(outGrid);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        grid.apply<hvdb::AllGridTypes>(op);
                     }
 
 #ifdef SESI_OPENVDB
@@ -565,7 +564,7 @@ SOP_OpenVDB_Resample::Cache::cookVDBSop(OP_Context& context)
                     op.xform = grid.constTransform();
                     op.xform.preMult(xform.getTransform().inverse());
 
-                    UTvdbProcessTypedGridReal(valueType, grid, op);
+                    grid.apply<hvdb::RealGridTypes>(op);
                     outGrid = op.outGrid;
                     outGrid->setTransform(grid.constTransform().copy());
 
@@ -577,13 +576,13 @@ SOP_OpenVDB_Resample::Cache::cookVDBSop(OP_Context& context)
 
                     if (curOrder == 0) {
                         hvdb::GridTransformOp<openvdb::tools::PointSampler> op(outGrid, xform);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(*vdb, op);
                     } else if (curOrder == 1) {
                         hvdb::GridTransformOp<openvdb::tools::BoxSampler> op(outGrid, xform);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(*vdb, op);
                     } else if (curOrder == 2) {
                         hvdb::GridTransformOp<openvdb::tools::QuadraticSampler> op(outGrid, xform);
-                        GEOvdbProcessTypedGridTopology(*vdb, op);
+                        hvdb::GEOvdbApply<hvdb::VolumeGridTypes>(*vdb, op);
                     }
 
 #ifdef SESI_OPENVDB
@@ -601,7 +600,7 @@ SOP_OpenVDB_Resample::Cache::cookVDBSop(OP_Context& context)
                     // If (and only if) the grid is vector-valued, apply the transform
                     // to each voxel's value.
                     VecXformOp op(xform.getTransform());
-                    UTvdbProcessTypedGridVec3(valueType, *outGrid, op);
+                    outGrid->apply<hvdb::Vec3GridTypes>(op);
                 }
             }
 
