@@ -35,7 +35,6 @@
 #include <PRM/PRM_Parm.h>
 #include <UT/UT_Interrupt.h>
 #include <UT/UT_UniquePtr.h>
-#include <UT/UT_Version.h>
 
 #include <hboost/algorithm/string/join.hpp>
 
@@ -557,7 +556,7 @@ getMaskFromGrid(const hvdb::GridCPtr& gridPtr, double isovalue = 0.0)
             maskGridPtr = gridPtr;
         } else {
             InteriorMaskOp op{isovalue};
-            UTvdbProcessTypedGridTopology(UTvdbGetGridType(*gridPtr), *gridPtr, op);
+            gridPtr->apply<hvdb::AllGridTypes>(op);
             maskGridPtr = op.outGridPtr;
         }
     }
@@ -705,18 +704,8 @@ SOP_OpenVDB_To_Polygons::Cache::cookVDBSop(OP_Context& context)
             for (; vdbIt; ++vdbIt) {
 
                 if (boss.wasInterrupted()) break;
-                //GEOvdbProcessTypedGridScalar(*vdbIt.getPrimitive(), mesher);
 
-                if (!GEOvdbProcessTypedGridScalar(*vdbIt.getPrimitive(), mesher)) {
-
-                    if (vdbIt->getGrid().type() == openvdb::BoolGrid::gridType()) {
-
-                        openvdb::BoolGrid::ConstPtr gridPtr =
-                            openvdb::gridConstPtrCast<openvdb::BoolGrid>(vdbIt->getGridPtr());
-
-                        mesher(*gridPtr);
-                    }
-                }
+                hvdb::GEOvdbApply<hvdb::ScalarGridTypes>(**vdbIt, mesher);
 
                 copyMesh(*gdp, mesher, boss, usePolygonSoup,
                     keepVdbName ? vdbIt.getPrimitive()->getGridName() : nullptr);
