@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file SOP_OpenVDB_Visualize.cc
 ///
@@ -55,7 +28,6 @@
 #include <PRM/PRM_Parm.h>
 #include <UT/UT_Interrupt.h>
 #include <UT/UT_VectorTypes.h> // for UT_Vector3i
-#include <UT/UT_Version.h>
 #include <UT/UT_UniquePtr.h>
 
 #include <algorithm>
@@ -65,12 +37,30 @@
 #include <vector>
 
 
-namespace std {
-template<> struct is_integral<openvdb::PointIndex32>: public true_type {};
-template<> struct is_integral<openvdb::PointIndex64>: public true_type {};
-template<> struct is_integral<openvdb::PointDataIndex32>: public true_type {};
-template<> struct is_integral<openvdb::PointDataIndex64>: public true_type {};
-}
+namespace {
+
+template <typename T>
+struct IsGridTypeIntegral
+    : std::conditional_t<   std::is_integral<T>::value
+                         || std::is_same<T,openvdb::PointIndex32>::value
+                         || std::is_same<T,openvdb::PointIndex64>::value
+                         || std::is_same<T,openvdb::PointDataIndex32>::value
+                         || std::is_same<T,openvdb::PointDataIndex64>::value
+                         , std::true_type
+                         , std::false_type>
+{
+};
+
+template <typename T>
+struct IsGridTypeArithmetic
+    : std::conditional_t<   IsGridTypeIntegral<T>::value
+                         || std::is_floating_point<T>::value
+                         , std::true_type
+                         , std::false_type>
+{
+};
+
+} // anonymous namespace
 
 namespace hvdb = openvdb_houdini;
 namespace hutil = houdini_utils;
@@ -691,7 +681,7 @@ private:
     GA_Offset createPoint(const openvdb::CoordBBox&, const UT_Vector3& color);
 
     template<typename ValType>
-    typename std::enable_if<std::is_integral<ValType>::value>::type
+    typename std::enable_if<IsGridTypeIntegral<ValType>::value>::type
     addPoint(const openvdb::CoordBBox&, const UT_Vector3& color, ValType s, bool);
 
     template<typename ValType>
@@ -699,7 +689,7 @@ private:
     addPoint(const openvdb::CoordBBox&, const UT_Vector3& color, ValType s, bool);
 
     template<typename ValType>
-    typename std::enable_if<!std::is_arithmetic<ValType>::value>::type
+    typename std::enable_if<!IsGridTypeArithmetic<ValType>::value>::type
     addPoint(const openvdb::CoordBBox&, const UT_Vector3& color, ValType v, bool staggered);
 
     void addPoint(const openvdb::CoordBBox&, const UT_Vector3& color, bool staggered);
@@ -931,7 +921,7 @@ TreeVisualizer::createPoint(const openvdb::CoordBBox& bbox,
 
 
 template<typename ValType>
-typename std::enable_if<std::is_integral<ValType>::value>::type
+typename std::enable_if<IsGridTypeIntegral<ValType>::value>::type
 TreeVisualizer::addPoint(const openvdb::CoordBBox& bbox,
     const UT_Vector3& color, ValType s, bool)
 {
@@ -949,7 +939,7 @@ TreeVisualizer::addPoint(const openvdb::CoordBBox& bbox,
 
 
 template<typename ValType>
-typename std::enable_if<!std::is_arithmetic<ValType>::value>::type
+typename std::enable_if<!IsGridTypeArithmetic<ValType>::value>::type
 TreeVisualizer::addPoint(const openvdb::CoordBBox& bbox,
     const UT_Vector3& color, ValType v, bool staggered)
 {
@@ -1239,17 +1229,14 @@ SOP_OpenVDB_Visualize::Cache::cookVDBSop(OP_Context& context)
                 if (meshing == MESH_HOUDINI) {
                     GridSurfacer surfacer(*gdp, static_cast<float>(iso),
                         static_cast<float>(adaptivity), false, &boss);
-                    GEOvdbProcessTypedGridScalar(*vdb, surfacer);
+                    hvdb::GEOvdbApply<hvdb::NumericGridTypes>(*vdb, surfacer);
                 }
 #endif
 
                 // draw tree topology
                 if (drawTree) {
                     TreeVisualizer draw(*gdp, treeParms, &boss);
-
-                    if (!GEOvdbProcessTypedGridTopology(*vdb, draw)) {
-                        GEOvdbProcessTypedGridPoint(*vdb, draw);
-                    }
+                    hvdb::GEOvdbApply<hvdb::AllGridTypes>(*vdb, draw);
                 }
 
                 if (showFrustum) {
@@ -1292,7 +1279,3 @@ SOP_OpenVDB_Visualize::Cache::cookVDBSop(OP_Context& context)
 
     return error();
 }
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
