@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @author FX R&D OpenVDB team
 
@@ -965,11 +938,7 @@ private:
 template<typename GridType, typename OpType, bool IsConst/*=false*/>
 struct GridProcessor {
     static inline void call(OpType& op, openvdb::GridBase::Ptr grid) {
-#ifdef _MSC_VER
-        op.operator()<GridType>(openvdb::gridPtrCast<GridType>(grid));
-#else
         op.template operator()<GridType>(openvdb::gridPtrCast<GridType>(grid));
-#endif
     }
 };
 
@@ -977,11 +946,7 @@ struct GridProcessor {
 template<typename GridType, typename OpType>
 struct GridProcessor<GridType, OpType, /*IsConst=*/true> {
     static inline void call(OpType& op, openvdb::GridBase::ConstPtr grid) {
-#ifdef _MSC_VER
-        op.operator()<GridType>(openvdb::gridConstPtrCast<GridType>(grid));
-#else
         op.template operator()<GridType>(openvdb::gridConstPtrCast<GridType>(grid));
-#endif
     }
 };
 
@@ -1081,7 +1046,3 @@ processTypedVectorGrid(GridPtrType grid, OpType& op)
 
 
 #endif // OPENVDB_MAYA_UTIL_HAS_BEEN_INCLUDED
-
-// Copyright (c) DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
